/***************************
	nfqueue.h
		2013.8.25
		by H.Nakano

**************************/
#ifndef __NFQUEUE_H
#define __NFQUEUE_H

#define RMemorySize  12
#define PathMax  		256
#define ip4addrLen 	16
#define ip6addrLen 	256
#define macLen 		20

#define 	RULES  "rule"
#define		INCOMING	1
#define		OUTGOING	2
#define		REASONbase	10
#define		RuleName	16
 
#ifndef IPPROTO_MH
#define IPPROTO_MH 135
#endif

typedef struct global_parm {
	int 	printLevel;
	int		rule_num;
	char 	zlogconf[PathMax];
	char 	dropconf[PathMax];
} global_parm_t; 

typedef struct nfque_parm {
	int 	queue_num;
	int    queue_end;
	char 	proto[8];
	char 	eth[ip4addrLen];
	char 	filter[8];
} nfque_parm_t; 

// host_info
typedef struct devip4addr {
	uint32_t flags;
	char name[ip4addrLen];
	char addr[ip4addrLen];
	char mask[ip4addrLen];
	char broadcast[ip4addrLen];
	char mac[macLen];
	struct devip4addr *next;
} devip4addr_t;

typedef struct devip6addr {
	uint32_t flags;
	char name[ip4addrLen];
	char addr[ip6addrLen];
	char mask[ip6addrLen];
	char mac[macLen];
	struct devip6addr *next;
} devip6addr_t;

typedef struct {
	int start;
	int end;
} port_range_t;

typedef struct {
	uint16_t type;
	uint16_t code_min;
	uint16_t code_max;
} icmp_t;

#define DropProtoNum 8

typedef struct drop_rule {
	int		rulenum;
	int		target; /* INCOMING or OUTGOING */
	int 	len;
	char   name[RuleName];
	char 	**eth; 
	uint8_t proto[DropProtoNum];
	icmp_t *icmp;
	int		*srcport;
	int  	*dstport;
	port_range_t *srcrange;
	port_range_t *dstrange;
} drop_rule_t;

typedef struct nfdef {
	int         	printlevel;
	int			 	dropnum;
	nfque_parm_t        *n; 
	drop_rule_t 		  *d;
	devip4addr_t 		  *host;
	struct nfq_handle   *h;  
	struct nfq_q_handle *qh;
} nfdef_t;

/* black reason */
#define  B_err    	1
#define  B_cache  	2
#define  B_in     	3
#define  B_out    	4
#define  B_payload	5

typedef struct ipt_info {
	uint32_t  verdict;
	int		   reason;
	uint32_t  hook;
	uint32_t  protocol;
	uint16_t  sport,dport;
	uint16_t  pktlen;
	uint16_t  icmp_type,icmp_code;
	uint16_t  offset; /* data offset */
	/* tcp */
	uint16_t  tcp_window;
 	uint8_t 	cwr;
	uint8_t 	ece;
	uint8_t 	urg;
	uint8_t 	ack;
	uint8_t 	psh;
	uint8_t 	rst;
	uint8_t 	syn;
	uint8_t 	fin;
	char rulename[RuleName];
	char srcip[ip4addrLen];
	char dstip[ip4addrLen];
	char inth[ip4addrLen];
	char outth[ip4addrLen];
//	char indev[ip4addrLen];
//	char outdev[ip4addrLen];
	char mac[macLen];
} ipt_info_t;

#endif 
