/************************
	addr_tools.c
		2013.9.17
		By H.Nakano
*************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdint.h>
#include <math.h>

static int check_cidr(int dat){
	int num;

	for(int i=2;i<= 28;i++){
		num = (int)pow(2.0, i);
		if (dat < num)
			return -1;
		if (dat == num)
			return 32 -i;
	}
	return -1;
}

static void get_addr(int flag,uint32_t num,char *addr,int cidr,uint32_t num2){
	if (flag == 1)
		sprintf(addr,"%d.%d.%d.%d\n",num >> 24,(num >> 16) & 0x00ff,
								    (num >> 8) & 0x0000ff,num & 0x000000ff);
	else if(flag == 2)
		sprintf(addr,"%d.%d.%d.%d/%d\n",num >> 24,(num >> 16) & 0x00ff,
								    (num >> 8) & 0x0000ff,num & 0x000000ff,cidr);

	else if(flag == 3)
		sprintf(addr,"%d.%d.%d.%d-%d.%d.%d.%d\n",num >> 24,(num >> 16) & 0x00ff,
								    (num >> 8) & 0x0000ff,num & 0x000000ff,
									 num2 >> 24,(num2 >> 16) & 0x00ff,
								    (num2 >> 8) & 0x0000ff,num2 & 0x000000ff);
}

void num2addr(uint32_t num,int range,char *val){
	int cidr;

	if (range ==0)
	 	get_addr(1,num,val,0,0);
	else{
		cidr = 0;
		if (num%2 ==0 && range >= 3)
			cidr =	check_cidr(range +1);

		if (cidr > 1)
			get_addr(2,num,val,cidr,0);
		else 
			get_addr(3,num ,val,0,num + range);
	 }
}

int get_cidr(char *dat){
	int num  = 0;
	char *buf = dat;
	while(isdigit(*buf)){
		num = (num * 10) + (*buf - '0');
		buf++;
		if (num >= 29)
		  return -1;
 	 }
	
	if (*buf != ' ' && *buf != '\t' && *buf != '\n' && *buf != '\0')
		return -1;
	return num;
}

int addr2num(char *dat,uint32_t *val,int *range){
	int num[4] = {0,0,0,0};
	int flag = 0;
	char *buf = dat;

	for(int i=0;i < 4;i++){
		while(isdigit(*buf)){
			num[i] = (num[i] * 10) + (*buf - '0');
			buf++;
			if (i ==0 && (num[i] <=0 || num[i] >= 256))
				return -1;
			else if(i >= 1 && num[i] >= 256)
				return -1;
 		 }

		if (i <3){
			if (*buf == '.')
				buf++;
			else
				return -2;
		 } else if (i== 3){
			if (*buf == '/'){
				flag =1;
				buf++;
			} else if (*buf == '-'){
				flag =2;
				buf++;
			} else if (*buf == ' ' || *buf == '\t' || *buf == '\0' || *buf == '\n') {
				flag = 0;
			} else return -3;
		 }
	}

	*val = num[0] << 24 | num[1] << 16 | num[2] << 8 | num[3];

	if (flag == 1){
		*range = get_cidr(buf);
		if (*range <= 1 || *range >= 29)
			return -10;
		*range = (int)pow(2.0, 32 - *range) -1;
	 } else if (flag == 2){
		uint32_t val2;
		int		 range2;
		if (addr2num(buf,&val2,&range2) != 0)
			return -11;
		if ((*val & 0xff000000) != (val2 & 0xff000000))
			return -12;
		*range = val2 - *val;
	 } else
		*range = 0;

//	printf("flag:%d :: range:%d \n",flag,*range);
	return flag;
}



