/*********************** 
	blackip.c
		2013.8.29
		By H.Nakano

		version 0.5
--		email: hnakano0157@gmail.com
--		copyright H.Nakano
--		License GPL version 3
***********************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <getopt.h>
#include <sys/types.h>

#include "hashdb.h"

void usage(){
	printf("usage\n");
	printf("\t -r,--host    server_address \n");
	printf("\t -p,--passwd  passwd (max 32 char) \n");
	printf("\t -a,--add  filename or ipaddress \n");
	printf("\t -d,--del  filename or ipaddress \n");
	printf("\t -c,--check  ipaddress \n");
	printf("\t -l,--list [1-255] : top number of ipaddress \n");
	printf("\t -t,--sum   : print total registered ipaddresses. \n");
	printf("\t -f,--file  filename : for --sum, --list\n");
	printf("\t\t(default: current-dir/blackip_sum.txt,blackip_xx.txt\n");
	printf("\t -s,--save  : save database. \n");
	printf("\t -u,--load  : load database. \n");
	printf("\t -n,--clean : clean all registered ipaddresses. \n");
	printf("\t -k,--stop  : stop the blackDB daemon program. \n");
	printf("\t -h,--help  : print this. \n\n");
	printf("\tex)\n");
	printf("\t blackip -r 127.0.0.1 -p 123456 -a ../blacklists/blacklist.txt\n");
	printf("\t blackip -r 127.0.0.1 -p 123456 -a 1.2.3.4\n");
	printf("\t blackip -r 127.0.0.1 -p 123456 -c 1.2.3.4\n");
	printf("\t blackip -r 127.0.0.1 -p 123456 -d 1.2.3.4\n");
	printf("\t blackip -r 127.0.0.1 -p 123456 -a 1.2.3.4-1.2.3.11\n");
	printf("\t blackip -r 127.0.0.1 -p 123456 -a 1.2.3.0/24\n");
	printf("\t blackip -r 127.0.0.1 -p 123456 -c 1.2.3.0/24\n");
	printf("\t blackip -r 127.0.0.1 -p 123456 --save\n");
	printf("\t blackip -r 127.0.0.1 -p 123456 --list 5 -f /home/nakano/blackip/blackip_5.txt\n");
	printf("\t blackip -r 127.0.0.1 -p 123456 --sum -f /home/nakano/blackip/blackip_sum.txt\n");
	printf("\t blackip -r 127.0.0.1 -p 123456 -k \n\n");
	printf("\t Version 0.5\n");

	exit(1);
}

/* tcp_client2.c */
extern int tcp_client(tcp_data_t *val,char *buf);
/* read_file.c */
extern int check_ipaddr_range(char *addr,ipaddr_range_t *rr);
extern ipaddr_range_t * read_file(FILE *fp, int*size); 
/* print_addr.c */
extern int save_sum(int size,int *data,char * filename);
extern int save_datalist(int size,int top,int *data,char *filename);

void ctl_db(int flag,char *host,char *passwd,char *val,char *savefile){
	int ret,num = 0;
	FILE *fp;
	tcp_data_t tdata;
	ipaddr_range_t *rr = NULL;
	char filename[PathMax];

	memset((void *)&tdata,0,sizeof(tcp_data_t));
	tdata.flag = flag;
	memcpy((void *)tdata.host,  host,strlen(host));
	memcpy((void *)tdata.passwd,passwd,strlen(passwd));

	switch(flag){
		case BLK_ADD:
		case BLK_DEL:
			fp = fopen(val,"r");
			if (fp == NULL){
				rr = (ipaddr_range_t *)malloc(sizeof(ipaddr_range_t));
				num = check_ipaddr_range(val,rr);
				if (num == -1){
					fprintf(stderr,"\t Error address (%s)\n",val);
					return;
				 }
				tdata.data = (void *)&rr;
				tdata.size = 0;
		 	} else {
				rr = read_file(fp,&num); 
				if (num == -1){
					fprintf(stderr,"\t Error address (%s)\n",val);
					return;
				 }
				fclose(fp);
				tdata.data = (void *)rr;
				tdata.size = num;
			}
			break;
		case BLK_CHECK:
			rr = (ipaddr_range_t *)malloc(sizeof(ipaddr_range_t));
			num = check_ipaddr_range(val,rr);
			if (num == -1){
				fprintf(stderr,"\t Error address (%s)\n",val);
				return;
			 }
			tdata.data = (void *)rr;
			tdata.size = 0;
			break;
		case  BLK_LIST:
			num = atoi(val);
			if (num < 0 || num >= 256){
				fprintf(stderr,"\t Error top [1-255] (%s)\n",val);
				return;
			 }
			tdata.top = num;
			tdata.size = 0;
			break;
	}

	/* tcp_client2.c */
 	ret = tcp_client(&tdata,filename);
	if (rr != NULL)
		free(rr);

	if (ret == -1)
		fprintf(stderr,"Error: send data\n");
	else if (ret == -2)
		fprintf(stderr,"Error: recv data\n");
	if(ret < 0){
		return;
	 }
	printf("result: %s\n", filename);

	if (strcmp(filename,Result_OK) != 0)
		return;

 	if (flag == BLK_LIST){
		if (tdata.data == NULL){
			fprintf(stderr,"non data\n");
			return;
		 }
		if (savefile != NULL)
			strcpy(filename,savefile);
		else 
			sprintf(filename,"blackip_%d.txt",num);	
		 
		ret = save_datalist(tdata.size,tdata.top,tdata.data,filename);
		if (ret == -1)
			fprintf(stderr,"Error: save list\n");
		free(tdata.data);

	} else if(flag == BLK_SUM){
		if (tdata.data == NULL){
			fprintf(stderr,"non data\n");
			return;
		 }

		if (savefile != NULL)
			strcpy(filename,savefile);
		else 
			strcpy(filename,"blackip_sum.txt");

		ret = save_sum(tdata.size,tdata.data,filename);
		if (ret == -1)
			fprintf(stderr,"Error: save sum\n");
		free(tdata.data);
	 }
}

int main (int argc, char **argv){

	const struct option longopt[] = {
    	{"help", no_argument, NULL, '?'},
    	{"host"  ,required_argument,NULL, 'r'},
    	{"passwd",required_argument,NULL, 'p'},
    	{"add",  required_argument, NULL, 'a'},
		{"del",  required_argument, NULL, 'd'},  
    	{"check",required_argument, NULL, 'c'}, 
    	{"list", required_argument, NULL, 'l'},
   		{"sum",  no_argument, NULL, 't'},
    	{"file", required_argument, NULL, 'f'},
  		{"save", no_argument, NULL, 's'},
		{"load", no_argument, NULL, 'u'},
   		{"clean",no_argument, NULL, 'n'},
  		{"stop", no_argument, NULL, 'k'},
    	{NULL, 0, NULL, 0}
     };

  	int c,flag = 0;
	char *val = NULL;
	char *savefile = NULL;
	char *passwd = NULL;
	char *host = NULL;

  	opterr = 0;
 	while ((c = getopt_long(argc, argv, "tnklsa:d:c:u:f:r:p:",longopt,NULL)) != -1){
     switch (c)
       {
     	case 'r': /* server */
			host = optarg;
        	break;
      	case 'p': /* passwd */
			passwd = optarg;
        	break;
      	case 'a': /* add */
        	flag = BLK_ADD;
			val = optarg;
        	break;
		case 'd': /* del */
        	flag = BLK_DEL;
			val = optarg;
			break;
 		case 'c': /* check */
        	flag = BLK_CHECK;
			val = optarg;
			break;
 		case 'l': /* list */
        	flag = BLK_LIST;
			val = optarg;
			break;
 		case 't': /* sum */
        	flag = BLK_SUM;
			break;
 		case 'f': /* savefile for sum,list */
			savefile = optarg;
			break;
 		case 's': /* save */
        	flag = BLK_SAVE;
			break;
		case 'u': /*  */
        	flag = BLK_LOAD;
			break;
 		case 'n': /* clean */
        	flag = BLK_CLEAN;
			break;
 		case 'k': /* kill */
        	flag = BLK_KILL;
			break;
      	default:
        	usage();
			break;
       }
	}

	if (flag == 0)
		usage();
	if (host == NULL || passwd == NULL){
		printf("Please use -h server -p passwd\n\n");
		usage();
	 }

	ctl_db(flag,host,passwd,val,savefile);
  	return 0;
}

