/************************
  	blackipd.c
		2013.8.26
		By H.Nakano
		version 0.5
--		email: hnakano0157@gmail.com
--		copyright H.Nakano
--		License GPL version 3
****************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <getopt.h>
#include <syslog.h>
#include <errno.h>

#include <glib.h>

#include "ipt_msg.h"
#include "hashdb.h"

#define  Pname "blackipd"

void usage(){
	printf("usage:  blackipd option -c confg-file \n");
	printf("\t -d,--daemon  (default no-daemon)\n");
	printf("\t -c,--config  config-file \n");
	
	printf("\tex)\n");
	printf("\t blackipd -d -c ../conf/blackdb.conf\n");
	printf("\t blackipd -c /var/lib/iptips/conf/blackdb.conf\n\n");
	printf("\t Version 0.5\n");
	exit(1);
}

/* hashdb.c */
extern int open_db(int flag,char *filename);
extern int close_db(int flag);
/* msgS.c */
extern int init_msg_server();
/* tcp_server2.c */
extern int tcp_server2(allowed_host_t *hosts);
/* addr_tools.c */
extern int addr2num(char *addr,uint32_t *val,int *range);

/* write_log.c */
extern void set_daemon();
extern void open_log(char *keyname,int facility);
extern int  open_zlog(char *filename,char *keys);
extern void close_log();
extern void writelog(int prio,char *fmt, ...);

typedef struct dbfile {
	int load;
	int save;
	char file[PathMax];
	char zlogconf[PathMax];
	int  nfqd;
	int  daemon;
	char nfqd_prg[PathMax];
	char nfqd_conf[PathMax];
} dbfile_t;

int check_dir(char *filename){
	gchar *dirname = g_path_get_dirname(filename);
	if (dirname == NULL){
		return -1;
	 }

	int ret = 0;
	if ( g_file_test(dirname,G_FILE_TEST_IS_DIR ) != TRUE){
		if (g_mkdir_with_parents(dirname,S_IRWXU | S_IXGRP| S_IRGRP | S_IROTH | S_IXOTH) < 0)
			ret = -1;
	}
	g_free(dirname);
	return ret;
}

uint32_t check_access(gchar **dat){
	uint32_t val = 0;
	char * name;

	for(int i=0; dat[i] != NULL; i++){
		name = g_ascii_strdown(dat[i],strlen(dat[i]));
		if (strcmp(name,"all") == 0)
			{ val = BLK_ALL; break; }
		else if (strcmp(name,"add") == 0)
			val |= BLK_ADD;
		else if (strcmp(name,"del") == 0)
			val |= BLK_ADD;
		else if (strcmp(name,"check") == 0)
			val |= BLK_CHECK;
		else if (strcmp(name,"list") == 0)
			val |= BLK_LIST;
		else if (strcmp(name,"sum") == 0)
			val |= BLK_SUM;
		else if (strcmp(name,"save") == 0)
			val |= BLK_SAVE;
		else if (strcmp(name,"load") == 0)
			val |= BLK_LOAD;
		else if (strcmp(name,"clean") == 0)
			val |= BLK_CLEAN;
		else if (strcmp(name,"kill") == 0)
			val |= BLK_KILL;
		else 
			{ val = 0; break;}
	 }
	return val;
}

allowed_host_t * get_config(char *filename,dbfile_t *dbfile){
	GKeyFile *gfile;
	gchar *mode;

  	gfile = g_key_file_new ();  
  	if (g_key_file_load_from_file(gfile,filename, 0,NULL) == FALSE){
		writelog(LOG_ERR,"Error: read file(%s)", filename);
		return NULL;
	}

	g_key_file_set_list_separator(gfile,',');

	if (g_key_file_has_group(gfile,"global") == FALSE){
		writelog(LOG_ERR,"Error: (%s) no global group", filename);
		return NULL;
	}
	memset((void *)dbfile,0,sizeof(dbfile_t));
	mode = g_key_file_get_string(gfile,"global","loadDB",NULL);
	if (mode){
		if (strcmp(g_ascii_strdown(mode,strlen(mode)),"on") ==0)
			dbfile->load = 1;
		free(mode);
	 }

	mode = g_key_file_get_string(gfile,"global","saveDB",NULL);
	if (mode){
		if (strcmp(g_ascii_strdown(mode,strlen(mode)),"on") ==0)
			dbfile->save = 1;
		free(mode);
	 }

	gchar *database = g_key_file_get_string(gfile,"global", "dbfile",NULL);
	if (database){
		check_dir(database);
		strcpy(dbfile->file,database);
		free(database);
	 } else {
		#define DBFileName "/var/lib/iptips/blackip.rdb"
		check_dir(DBFileName);
		strcpy(dbfile->file,DBFileName);
	 }

	if( check_dir(dbfile->file) < 0){
		writelog(LOG_ERR,"Error: mkdir (%s)", dbfile->file);
	}	

	/* zlog */
	database = g_key_file_get_string(gfile,"global", "zlogconf",NULL);
	if (database){
		if (g_file_test(database,G_FILE_TEST_IS_REGULAR) == TRUE)  
			strcpy(dbfile->zlogconf,database);
		else
			writelog(LOG_ERR,"Error: no zlogconf (%s)",database);
		free(database);
	 }

	/* NFQUEUE */
	if (g_key_file_has_group(gfile,NFQname) == FALSE){
		writelog(LOG_ERR,"Error: (%s) no nfqueue group", filename);
	} else {
		int flag = 0;
		mode = g_key_file_get_string(gfile,NFQname,"use",NULL);
		if (mode){
			if (strcmp(g_ascii_strdown(mode,strlen(mode)),"on") ==0)
				dbfile->nfqd = 1;
			free(mode);
		 }

		if (dbfile->nfqd == 1){
			mode = g_key_file_get_string(gfile,NFQname,"daemon",NULL);
			if (mode){
				if (strcmp(g_ascii_strdown(mode,strlen(mode)),"on") ==0)
					dbfile->daemon = 1;
				free(mode);
			 }

			mode = g_key_file_get_string(gfile,NFQname,"prg",NULL);
			if (mode){
				if (g_file_test(mode,G_FILE_TEST_IS_EXECUTABLE) == TRUE){
					strcpy(dbfile->nfqd_prg,mode);
			 		flag = 1;
				 }
				free(mode);
			 }
			mode = g_key_file_get_string(gfile,NFQname,"conf",NULL);
			if (mode){
				if (g_file_test(mode,G_FILE_TEST_IS_REGULAR) == TRUE){ 
					strcpy(dbfile->nfqd_conf,mode);
					flag ++;
				 }
				free(mode);
			 }

			if (flag != 2)
		  		dbfile->nfqd = 0;
		 }
	}

	/* set hosts */	
	int  i,num;
	gsize size;
	allowed_host_t *hosts;
	gchar *addr,*passwd,**access;
	gchar **groups,*group;

	groups = g_key_file_get_groups(gfile,&size);
	if (size <= 2){
		if (groups != NULL)
			g_strfreev(groups);
		g_key_file_free(gfile);
		writelog(LOG_ERR,"Error: groups");
		return NULL;
	 }

	hosts = (allowed_host_t *)calloc(size -1,sizeof(allowed_host_t));
	num =0;
	for(i=0;i < size;i++){
		group = groups[i];
		if (strcmp(group,"global") == 0 || strcmp(group,NFQname) == 0)
			continue;

		if (g_key_file_has_group(gfile,group) == FALSE){
			writelog(LOG_ERR,"Error: group (%s)",group);
			continue;
		}

		addr = g_key_file_get_string(gfile,group,"addr",NULL);
		if (addr){
			uint32_t val;
			int      range;
			if (addr2num(addr,&val,&range) != 0){
				writelog(LOG_ERR,"error:(%s)",addr);
				num = -1;
		     } else
				 strcpy(hosts[num].addr, g_strstrip(addr)); 
			g_free(addr); 
		 } else {
			writelog(LOG_ERR,"error: %s addr",group);
			num = -1;
		 }

		passwd = g_key_file_get_string(gfile,group,"passwd",NULL);
		if (passwd){
			strcpy(hosts[num].passwd,g_strstrip(passwd));
			g_free(passwd);
		 } else {
			writelog(LOG_ERR,"error: %s passwd",group);
			num = -1;
		 }

		access = g_key_file_get_string_list(gfile,group,"access",NULL,NULL);
		if (access){
			hosts[num].access = check_access(access); 
			g_free(access);
			if (hosts[num].access == 0){
				writelog(LOG_ERR,"error: %s access",group);
				num = -1;
		      }
		 } else {
			writelog(LOG_ERR,"error: %s access",group);
			num = -1;
		 }

		if (num < 0)
			break;
		num ++;
	 }

	g_strfreev(groups);
	g_key_file_free (gfile);

	if (num < 0){
		free(hosts);
		return NULL;
	 }
	return hosts;
}

/* msgS.c :: thread */
void *msg_server(){
	int ret = init_msg_server();
	if (ret == -1){
		writelog(LOG_ERR,"Error: msg server: start");
		exit(1);
	 }	
	return NULL;
}

void *tcp_server(void *val){
	int ret = tcp_server2((allowed_host_t *)val);
	if (ret == -1){
		writelog(LOG_ERR,"Error: tcp_server: start");
		exit(1);
	 } 
	return NULL;
}

int main(int argc,char **argv){
  	int c,ret;
	int dflag = 0;
	char *filename = NULL;
	dbfile_t dbfile;
	allowed_host_t *hosts;

	const struct option longopt[] = {
    	{"help", no_argument, NULL, '?'},
    	{"daemon", no_argument, NULL, 'd'},
    	{"config", required_argument, NULL, 'c'},
    	{NULL, 0, NULL, 0}
     };

  	opterr = 0;
 	while ((c = getopt_long(argc, argv, "dc:",longopt,NULL)) != -1){
     switch (c)
       {
      	case 'd': // daemon
        	dflag = 1;
        	break;
      	case 'c': // config
        	filename = optarg;
        	break;
      	default:
        	usage();
			break;
       }
	}

	if (filename == NULL){
		writelog(LOG_ERR,"No config-file");
		usage();
		return -1;
	 } 

	hosts = get_config(filename,&dbfile);
	if (hosts == NULL)
		return -1;

	open_zlog(dbfile.zlogconf,Pname);

	if (dflag == 1){
		if(daemon(0,0) == -1){
			writelog(LOG_ERR,"failed to launch daemon.");
			free(hosts);
      		return -1;
		 }
	   writelog(LOG_INFO,"blackipd daemon started.");
	}

	ret = nice(-18);
	writelog(LOG_INFO,"nice (%d)",ret);

	ret = open_db(dbfile.load,dbfile.file);
	if (ret != Check_OK)
		writelog(LOG_ERR,"Error: load DB file (%s)",dbfile.file);

	if (dbfile.nfqd == 1){
  		pthread_t thread1, thread2;
		char *pargs[4];
 
 		if (dbfile.daemon == 1){
			pargs[0] = "-d";
			pargs[1] = "-c";
			pargs[2] = dbfile.nfqd_conf;
			pargs[3] = NULL;
		} else {
			pargs[0] = "-c";
			pargs[1] = dbfile.nfqd_conf;
			pargs[2] = NULL;
		 } 
		errno = 0;
		pid_t child = fork();
		if (child == 0) {
			execve(dbfile.nfqd_prg,pargs,NULL);
			writelog(LOG_INFO,"fork (%s) started.",dbfile.nfqd_prg);
		} else if (child == -1) {
			writelog(LOG_ERR, "Error: (%s) fork (%s)",dbfile.nfqd_prg,strerror(errno));
			exit(1);
	    }	

  		pthread_create(&thread1, NULL,&msg_server,NULL);
   		pthread_create(&thread2, NULL,&tcp_server,(void *)hosts);

  		pthread_join(thread2,NULL);
   		pthread_join(thread1,NULL);

		/* nfqd stop */
		system("killall nfqd -s 10");

	} else {
   		tcp_server((void *)hosts);
	 }

	ret = close_db(dbfile.save);
	if (ret != Check_OK)
		writelog(LOG_ERR,"Error: save DB file (%s)",dbfile.file);

	writelog(LOG_INFO,"blackipd daemon stopped.");
	free(hosts);
	close_log();
	return 0;
}


