/************************
  	drop_rule.c
		2013.10.08
		By H.Nakano
	version 01.
****************************/
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <stdint.h>
#include <sys/types.h>
#include <unistd.h>

#include <syslog.h>
#include <errno.h>
#include <netinet/in.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>

#include <glib.h>

#include "nfqueue.h"

/* parse_name.c */
extern uint8_t parse_protocol(const char *name);
extern int parse_icmp(const char *icmptype,icmp_t code[]);

/* check_port.c */
extern int check_ports(int num,int *val);
extern port_range_t *check_port_range(char *dat);

extern void writelog(int prio,char *fmt, ...);

uint32_t check_dpport(drop_rule_t *d,ipt_info_t *ipt_info){
	uint32_t verdict = NF_ACCEPT; 
	int flag = 0;

	if (ipt_info->hook == NF_IP_PRE_ROUTING || ipt_info->hook == NF_IP_LOCAL_IN){
		if (d->target != INCOMING)
			return NF_ACCEPT; 

		if (d->eth != NULL && ipt_info->inth != NULL){
			flag = 0;
			for(int j=0; d->eth[j] != NULL;j++){
				if (strcasecmp(ipt_info->inth,d->eth[j]) == 0)
				  { flag = 1; break; }
			 }
			if (flag == 0)
				return NF_ACCEPT;
		}
	 } else if (ipt_info->hook == NF_IP_POST_ROUTING || ipt_info->hook == NF_IP_LOCAL_OUT){
		if (d->target != OUTGOING)
			return NF_ACCEPT; 
		if (d->eth != NULL || ipt_info->outth != NULL){
			flag = 0;
			for(int j=0; d->eth[j] != NULL;j++){
				if (strcasecmp(ipt_info->outth,d->eth[j]) == 0)
				  { flag = 1; break; }
			 }
			if (flag == 0)
				return NF_ACCEPT;
		 }
	 }
 	else if (ipt_info->hook == NF_IP_FORWARD){
		return NF_ACCEPT;
	} 
		
		for(int j=0; d->proto[j] != 0xff;j++){
			if (d->proto[j] == ipt_info->protocol)
			{ flag = 2; break; }
		}

		if (flag != 2)
			return NF_ACCEPT;

		if (ipt_info->protocol == 1){
				if( d->icmp != NULL){
					for(int j=0; d->icmp[j].type != 0xfe;j++){
						if(	ipt_info->icmp_type == d->icmp[j].type && 
					    	ipt_info->icmp_code >= d->icmp[j].code_min &&
					    	ipt_info->icmp_code <= d->icmp[j].code_max)
						 	{verdict = NF_DROP; break;}
				  	}
			   } else
				return NF_ACCEPT;

		 } else {

			if (d->srcport != NULL){
				for(int j=0; d->srcport[j] != 0;j++){
					if (ipt_info->sport == d->srcport[j]){
					 flag = 3; break; }
				 }
				if (flag != 3)
				return NF_ACCEPT;
			 }

			if (d->dstport != NULL){
				for(int j=0; d->dstport[j] != 0;j++){
					if (ipt_info->dport == d->dstport[j]){
					 flag = 4; break; }
				 }
				if (flag != 4)
				return NF_ACCEPT;
			 } 

			if (d->srcrange != NULL){
				if (ipt_info->sport >= d->srcrange->start && ipt_info->sport <= d->srcrange->end)
					 flag = 5; 
				else
					return NF_ACCEPT;
			 }

			if (d->dstrange != NULL){
				if (ipt_info->dport >= d->dstrange->start && ipt_info->dport <= d->dstrange->end)
					 flag = 6; 
				else
					return NF_ACCEPT;
			 }


		   if (flag >= 3)
				verdict = NF_DROP;
	  }	


	if (verdict == NF_DROP){
	//	writelog(LOG_INFO,"ICMP flag=%d type=%u code=%u",flag,ipt_info->icmp_type,ipt_info->icmp_code );
		ipt_info->reason = REASONbase + d->rulenum;
		strcpy(ipt_info->rulename,d->name);
	 }
	return verdict;
}

void set_dparm(int rulemax,nfque_parm_t *parm,drop_rule_t *dparm){
	int ret;

	for(int i=0; i < rulemax;i++){
		if (dparm[i].target == OUTGOING){
			if(strcmp(parm->filter,"mangle") != 0 && strcmp(parm->filter,"output") != 0)
				{ dparm[i].rulenum = -1; continue; }
		 } else {
			if(strcmp(parm->filter,"raw") != 0 && strcmp(parm->filter,"input") != 0)
				{ dparm[i].rulenum = -1; continue; }
		 }

		if (parm->eth != NULL && dparm[i].eth != NULL){
			ret = 0;
			for(int j=0; dparm[i].eth[j] != NULL;j++){
				if (strcasecmp(parm->eth,dparm[i].eth[j]) == 0)
				  { ret = 1; break; }
			 }
			if (ret == 0)
				{ dparm[i].rulenum = -1; continue; }
		 }

		if (parm->proto == NULL || strcmp(parm->proto,"all") == 0){
			continue;
		 } 
		ret = 0;
		if (parm->proto[0] == '!'){
				char *proto = &parm->proto[1];
				if (strcmp(proto,"icmp") == 0 && dparm[i].icmp != NULL){
					ret = 1;  
			 	} else {
					if (dparm[i].proto[0] != 0xff){ 
						ret = 0;
						uint8_t pp = parse_protocol(proto);
						for(int j=0; dparm[i].proto[j] != 0xff;j++){
							if (pp != dparm[i].proto[j])
							 { ret = 1; break; } 
						  }
				      }
				}
				if (ret == 0)
					dparm[i].rulenum = -1;
				continue;
		 } 

		if (strcmp(parm->proto,"icmp") == 0){
			if (dparm[i].icmp != NULL)
				ret= 1;
	   } else {
			if (dparm[i].proto[0] != 0xff){
				uint8_t pp = parse_protocol(parm->proto);
				for(int j=0; dparm[i].proto[j] != 0xff;j++){
					if (pp == dparm[i].proto[j]) 
					{ ret= 1; break; }
				 }
			}
		 }

		if (ret == 0)
			dparm[i].rulenum = -1;
	}
}

void d_free(int num,drop_rule_t *dparm){
	for(int i=0;i < num;i++){
		if (dparm[i].eth != NULL)
			g_strfreev(dparm[i].eth);

		if (dparm[i].icmp != NULL)
			free(dparm[i].icmp);
	
		if (dparm[i].srcport != NULL)
			g_free(dparm[i].srcport);
		if (dparm[i].dstport != NULL)
			g_free(dparm[i].dstport); 
	
		if (dparm[i].srcrange != NULL)
			free(dparm[i].srcrange);
		if (dparm[i].dstrange != NULL)
			free(dparm[i].dstrange);
	 }
	free(dparm);
}

drop_rule_t *get_droprule(char *filename,int *maxrule){
	int  i,num;
	drop_rule_t *dparm;
	GKeyFile *gfile;
	GError *error = NULL;
	gsize  slen,dlen,size;
	gchar **groups,*group,*val;
	gchar **val2;

  	gfile = g_key_file_new();  
  	if (g_key_file_load_from_file(gfile,filename, 0,NULL) == FALSE){
		writelog(LOG_ERR,"Error: read file(%s)", filename);
		return NULL;
	}

	g_key_file_set_list_separator(gfile,',');

	groups = g_key_file_get_groups(gfile,&size);
	if (size <= 0){
		if (groups != NULL)
			g_strfreev(groups);
		g_key_file_free(gfile);
		writelog(LOG_ERR,"Error: groups");
		return NULL;
	 }
	dparm = (drop_rule_t *)calloc(size,sizeof(drop_rule_t));

	num =0;
	for(i=0;i < size;i++){
		group = groups[i];
		if (strcmp(group,"global") == 0)
			continue;
		
		if (g_key_file_has_group(gfile,group) == FALSE){
			writelog(LOG_ERR,"Error: group (%s)",group);
			continue;
		}

		val = g_key_file_get_string(gfile,group,"target",NULL);
		if(strcmp(g_ascii_strdown(val,-1),"outgoing") == 0)
			dparm[num].target = OUTGOING;
		else
			dparm[num].target = INCOMING;
		free(val);
		dparm[num].rulenum = i;

		val = g_key_file_get_string(gfile,group,"name",NULL);
		if (val != NULL){
			strcpy(dparm[num].name,g_strstrip(val));
			free(val);
		 } else{
			sprintf(dparm[num].name,"rule-%d",num +1);
		 }

		dparm[num].eth = g_key_file_get_string_list(gfile,group,"eth",NULL,NULL);
		val2 = g_key_file_get_string_list(gfile,group,"proto",NULL,NULL);
		for (int j=0;j < DropProtoNum;j++)
				dparm[num].proto[j] = 0xff;

		if (val2 != NULL){
			uint8_t ret;	
			for (int j=0;val2[j] != NULL;j++){
				ret = parse_protocol(val2[j]);
				if (ret >= 0)
					dparm[num].proto[j] = ret;
				else{
					writelog(LOG_ERR,"Error: (%s) (%s) protocol",filename,group);
					num = -1;
					break;
				 }
			 }
			g_strfreev(val2);
		 }
	
		val2  = g_key_file_get_string_list(gfile,group,"icmp",NULL,NULL);
		if (val2 != NULL){
			dparm[num].proto[0] = 1;
			int ret;
			int cc = 1;
			dparm[num].icmp = (icmp_t *)malloc(RMemorySize*sizeof(icmp_t));
			for (int j=0;j < RMemorySize;j++)
				dparm[num].icmp[j].type = 0xfe;

			for (int j=0;val2[j] != NULL;j++){
				if (j >= cc*RMemorySize){
					cc ++;
					dparm[num].icmp = (icmp_t *)realloc(dparm[num].icmp,RMemorySize*cc*sizeof(icmp_t));
					for (int i=0;i < RMemorySize;i++)
						dparm[num].icmp[RMemorySize*(cc-1) + i].type = 0xfe;
				 }
				ret = parse_icmp(val2[j],&dparm[num].icmp[j]);
				if (ret < 0){
					writelog(LOG_ERR,"Error: (%s) (%s) icmp",filename,group);
					num = -1;
					break;
				 }
			 }
			g_strfreev(val2);
		 } 
			
		dparm[num].len = g_key_file_get_integer(gfile,group,"len",NULL);

			error = NULL;
			dparm[num].srcport = g_key_file_get_integer_list(gfile,group,"srcport",&slen,&error);
			if (error != NULL){
				dparm[num].srcport = NULL;
			   if( error->code != G_KEY_FILE_ERROR_KEY_NOT_FOUND){
					val = g_key_file_get_string(gfile,group,"srcport",NULL);
					dparm[num].srcrange = check_port_range(val);
					free(val);
					if (dparm[num].srcrange == NULL){
			 	 		writelog(LOG_ERR,"Error: (%s) %s srcport",filename,group);
						g_error_free(error);
				 		num = -1; break;
			     	  } 
				 }
				g_error_free(error);
				error = NULL;
		    } else {

				if (dparm[num].srcport != NULL){
					if (check_ports(slen,dparm[num].srcport) < 0){
						writelog(LOG_ERR,"Error: srcport(%s) %s",filename,group);
						num = -1;
					 }
				 }else
						dparm[num].srcport[slen] = 0;
			}

			dparm[num].dstport = g_key_file_get_integer_list(gfile,group,"dstport",&dlen,&error);
			if (error != NULL){
				dparm[num].dstport = NULL;
				if(error->code != G_KEY_FILE_ERROR_KEY_NOT_FOUND){
					val = g_key_file_get_string(gfile,group,"dstport",NULL);
					dparm[num].dstrange = check_port_range(val);
					free(val);
					if (dparm[num].dstrange == NULL){
			 	 		writelog(LOG_ERR,"Error: (%s) %s dstport",filename,group);
						g_error_free(error);
				 		num = -1; break;
			          } 
				 }
				g_error_free(error);
				error = NULL;
			 } else {
				if (dparm[num].dstport != NULL){
					if (check_ports(dlen,dparm[num].dstport) < 0){
						writelog(LOG_ERR,"Error: dstport(%s) %s",filename,group);
						num = -1; 
					}
				} else
					dparm[num].dstport[dlen] = 0;
			 }

		if (num < 0)
			break;
		num ++;
	}

	g_strfreev(groups);
	g_key_file_free(gfile);
	if (num > 0){
		*maxrule = num;
		return dparm;
	} else {
	    d_free(i,dparm);
		return NULL;
	}
}

