/*****************
	getipaddr.c
		2013.8.15
		By H.Nakano

**********************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <syslog.h>

#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netdb.h>
#include <ifaddrs.h>
#include <linux/if.h>

#include "nfqueue.h"

extern void writelog(int prio,char *fmt, ...);

void print_hostinfo(devip4addr_t *addr){
	devip4addr_t *n;
	char dat[1024];
	for (n = addr; n != NULL ; n = n->next){
		sprintf(dat,"(%s) ipaddr:%s netmask:%s broadcast:%s ",n->name,n->addr,n->mask,n->broadcast);
		if (strcmp(n->name, "lo") != 0)
			sprintf(&dat[strlen(dat)],"mac:%s ",n->mac);

		sprintf(&dat[strlen(dat)],"%s %s %s %s %s %s %s", 
			n->flags & IFF_UP ? "UP" : "DOWN",
			n->flags & IFF_RUNNING   ? "RUNNING" : "NOT_RUNNING",
			n->flags & IFF_LOOPBACK  ? "LOOPBACK" : ""	,
			n->flags & IFF_DYNAMIC   ? "DYNAMIC" : "",
			n->flags & IFF_BROADCAST ? "BROADCAST" : "",
			n->flags & IFF_MULTICAST ? "MULTICAST" : "",
			n->flags & IFF_PROMISC   ? "PROMISC" : "");
		writelog(LOG_INFO,dat);
	 }
}

void free_hinfo(devip4addr_t *addr){

	devip4addr_t *tt = addr;
	while (tt != NULL){
  		devip4addr_t *n = tt;
   		tt = n->next;
   		free((void *)n);
	}
}

char *get_dev_ipaddr(devip4addr_t *addr,char *devname){
	devip4addr_t *n;
	if (addr == NULL)
		return NULL;

	for (n = addr; n != NULL; n = n->next){
		if (strcmp(n->name,devname) == 0)
			return n->addr;
	 }
	return NULL;
}

int get_mac_addr(char *devname,char *mac){
	int ret;
    struct ifreq s;
	if (strcmp(devname,"lo") == 0)
		return 0;
	errno = 0;
  	int fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
	if (fd < 0){
		writelog(LOG_ERR,"Error:socket open IPPROTO_IP (%s)",strerror(errno));
		return 1;
	 }
  	strcpy(s.ifr_name, devname);
 	 if (ioctl(fd, SIOCGIFHWADDR, &s) == 0) {
    	int i;
    	for (i = 0; i < 6; i++){
			  snprintf(mac + i*3,4,"%02x:",(unsigned char)s.ifr_hwaddr.sa_data[i]);
		 }
		mac[17] = '\0';
		ret = 0;
     }
	else {
		writelog(LOG_ERR,"Error:ioctl SIOCGIFHWADDR (%s)",strerror(errno));
		ret = 1;
	 }
	if (close(fd) < 0)
		writelog(LOG_ERR,"Error:socket close: SIOCGIFHWADDR (%s)",strerror(errno));
	return ret;
}

/* sys/socket.h */
int get_dev_flags(char *devname,uint32_t *flags){
	int ret;
    struct ifreq s;
  	int fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
	if (fd < 0){
		writelog(LOG_ERR,"Error:socket open IPPROTO_IP (%s)",strerror(errno));
		return 1;
	 }
	
  	strcpy(s.ifr_name, devname);
	ret = ioctl(fd,SIOCGIFFLAGS, &s);
    if (ret == 0)
       *flags = s.ifr_flags;
	else
		writelog(LOG_ERR,"Error:ioctl SIOCGIFCONF (%s)",strerror(errno));
	if (close(fd) < 0)
		writelog(LOG_ERR,"Error:socket close: SIOCGIFCONF (%s)",strerror(errno));
	return ret;
}

int get_devip4addr(devip4addr_t **ip4addr){
 	struct ifaddrs *ifaddr, *ifa;
	devip4addr_t *head; 
	char ipaddr[ip4addrLen],netmask[ip4addrLen],broadcast[ip4addrLen]; 
	int first = 1;

       if (getifaddrs(&ifaddr) == -1) {
           writelog(LOG_ERR,"Error:getifaddrs (%s)",strerror(errno));
           return -1;
         }

		*ip4addr = (devip4addr_t *)malloc(sizeof(devip4addr_t));
		head = *ip4addr;

       for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
         	if (ifa->ifa_addr == NULL || ifa->ifa_addr->sa_family != AF_INET)
           	continue;

         	inet_ntop(AF_INET,&((struct sockaddr_in *)ifa->ifa_addr)->sin_addr,ipaddr, ip4addrLen);
          	inet_ntop(AF_INET,&((struct sockaddr_in *)ifa->ifa_netmask)->sin_addr, netmask, ip4addrLen);
	      	inet_ntop(AF_INET,&((struct sockaddr_in *)ifa->ifa_ifu.ifu_broadaddr)->sin_addr,broadcast, ip4addrLen);
			
			if (first == 1){
				head->flags = ifa->ifa_flags;
				strcpy(head->name, ifa->ifa_name);
				strcpy(head->addr, ipaddr);
				strcpy(head->mask, netmask);
				strcpy(head->broadcast,broadcast);
				get_mac_addr(ifa->ifa_name,head->mac);

				head->next = NULL;
				first = 0;
			 } 
		    else {
				devip4addr_t *addr = (devip4addr_t *)malloc(sizeof(devip4addr_t));
				if (! addr){
						writelog(LOG_ERR,"Error:malloc struct devip4addr (%s)",strerror(errno));
					  	continue;
					}
				addr->flags = ifa->ifa_flags;
				strcpy(addr->name, ifa->ifa_name);
				strcpy(addr->addr, ipaddr);
				strcpy(addr->mask, netmask);
				strcpy(addr->broadcast,broadcast);
				get_mac_addr(ifa->ifa_name,addr->mac);
				addr->next = NULL;
				head->next = addr;
				head = addr;
			 }

        }
      freeifaddrs(ifaddr);
      return 0;
}

int get_devip6addr(devip6addr_t **ip6addr){
 	struct ifaddrs *ifaddr, *ifa;
	devip6addr_t *head; 
	char ipaddr[ip6addrLen],netmask[ip6addrLen]; 
	int first = 1;

       if (getifaddrs(&ifaddr) == -1) {
            writelog(LOG_ERR,"Error:getifaddrs (%s)",strerror(errno));
            return -1;
         }

		*ip6addr = (devip6addr_t *)malloc(sizeof(devip6addr_t));
		head = *ip6addr;

       for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
         	if (ifa->ifa_addr == NULL || ifa->ifa_addr->sa_family != AF_INET)
           	continue;

         	inet_ntop(AF_INET,&((struct sockaddr_in6 *)ifa->ifa_addr)->sin6_addr,ipaddr, ip6addrLen);
          	inet_ntop(AF_INET,&((struct sockaddr_in6 *)ifa->ifa_netmask)->sin6_addr, netmask, ip6addrLen);
				
			if (first == 1){
				head->flags = ifa->ifa_flags;
				strcpy(head->name, ifa->ifa_name);
				strcpy(head->addr, ipaddr);
				strcpy(head->mask, netmask);
				get_mac_addr(ifa->ifa_name,head->mac);
				head->next = NULL;
				first = 0;
			 } 
			else {
				devip6addr_t *addr = (devip6addr_t *)malloc(sizeof(devip6addr_t));
				if (! addr){
					writelog(LOG_ERR,"Error:malloc struct devip4addr (%s)",strerror(errno));
					continue;
				}
				addr->flags = ifa->ifa_flags;
				strcpy(addr->name, ifa->ifa_name);
				strcpy(addr->addr, ipaddr);
				strcpy(addr->mask, netmask);
				get_mac_addr(ifa->ifa_name,addr->mac);
				addr->next = NULL;
				head->next = addr;
				head = addr;
			 }
        }
      freeifaddrs(ifaddr);
      return 0;
}


