/*********************

 	hashdb.c
		2013.8.20
		H.Nakano
	
**************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <syslog.h>
#include <glib.h>

#include "ipt_msg.h"
#include "hashdb.h"

extern void writelog(int prio,char *fmt, ...);

static int  CacheCount = 0;
static uint32_t cachedAddr[MaxCache];
static int  GCacheCount = 0;
static uint32_t GcachedAddr[MaxCache];

static int Changed = 0;
static GHashTable* lip[topSize];
char *Val = "B";
char dbfilename[PathMax];

void clean_nfcache(int flag){

	if (flag == BLK_DEL || flag == BLK_CLEAN){
		memset((void *)cachedAddr,0,MaxCache*sizeof(uint32_t));
		CacheCount = 0;
	}

	if (flag == BLK_ADD || flag == BLK_CHkADD || flag == BLK_CLEAN){
		memset((void *)GcachedAddr,0,MaxCache*sizeof(uint32_t));
		GCacheCount = 0;
	 }
}

int check_cache(uint32_t ipaddr){
	int i;

	for(i=0; i < MaxCache;i++){
		if (cachedAddr[i] == 0) 
			break;
		if (cachedAddr[i] == ipaddr)
			return  BlackON;
	 }

	for(i=0; i < MaxCache;i++){
		if (GcachedAddr[i] == 0) 
			break;
		if (GcachedAddr[i] == ipaddr)
			return BlackOFF;
	 }
	return -1;
}

void set_cache(int ret,uint32_t ipaddr){
	if (ret == BlackON){
		cachedAddr[CacheCount++] = ipaddr;
		if (CacheCount >= MaxCache)
		  CacheCount = 0;

	} else {
		GcachedAddr[GCacheCount++] = ipaddr;
		if (GCacheCount >= MaxCache)
		  GCacheCount = 0;
	}
}

void clean_db(){
	for(int i=0;i < topSize;i++)
		lip[i] = NULL;
}

void init_db(int top){
	lip[top] = g_hash_table_new_full( g_direct_hash, g_direct_equal,NULL,NULL);
}

void clear_db(){
	for(int i=0;i < topSize;i++){
		if (lip[i] != NULL && g_hash_table_size(lip[i]) >= 1) 
			g_hash_table_remove_all(lip[i]);
	 }
	Changed = 1;
}

void destroy_db(){
	int i;
	for(i=0;i < topSize;i++){
		if (lip[i] != NULL) 
			 g_hash_table_destroy(lip[i]);
	 }
}

void add_db(uint32_t val,int range){
	int top = val >>24;
	int num = val & 0x00ffffff;
	if (lip[top] == NULL) 
		init_db(top);
	//	printf("%d %d %d\n",top,num,range);
	if (range == 0){
			g_hash_table_insert(lip[top], GINT_TO_POINTER(num), Val );
	} else {
		for (int i = num;i <= num +range; i++)
        	g_hash_table_insert(lip[top], GINT_TO_POINTER(i), Val );
	}
	Changed = 1;
}

int del_db(uint32_t val,int range){
	int top = val >>24;
	if (lip[top] == NULL) 		
		return Check_Fail;
	int num = val & 0x00ffffff;
	//	printf("%d %d %d\n",top,num,range);

	int ret = g_hash_table_remove( lip[top], GINT_TO_POINTER(num));
	if (range > 0){
		for (int i = num +1;i <= num +range; i++)
        	ret = g_hash_table_remove(lip[top], GINT_TO_POINTER(i));
	 }
	if (ret == TRUE){
		Changed = 1;
		return Check_OK;
	} else
		return Check_Fail;
}

int check_db(uint32_t val,int range){
	int top = val >>24;
	if (lip[top] == NULL || g_hash_table_size(lip[top]) == 0)
		return BlackOFF;

	int ret;
	if (range == 0){
		ret = check_cache(val);
		if (ret >= 0)
			return ret;
	 }

	int num = val & 0x00ffffff;
	ret = g_hash_table_lookup_extended (lip[top], GINT_TO_POINTER(num), NULL, NULL);
	if (range > 0){
		int ff = ret;
		for (int i=1; i<= range;i++){
			ff = g_hash_table_lookup_extended (lip[top], GINT_TO_POINTER(num + i), NULL, NULL);
			if (ff != ret)
				return BlackPart;
		 }
	 }

	set_cache(ret,val);
	if (ret == TRUE)
		return BlackON;
	else
		return BlackOFF;
}

int *get_datalist(int top,int *val){
	if (lip[top] == NULL || g_hash_table_size(lip[top]) == 0){
		*val = 0;
		return NULL;
	 }
	int size = g_hash_table_size(lip[top]);
	GHashTableIter iter;
	gpointer key, value;
	int *data = (int *)malloc(size*sizeof(int));
	if (data == NULL){
		*val = -1;
		return NULL;
	 }

	int i = 0;
	g_hash_table_iter_init (&iter, lip[top]);
 	while (g_hash_table_iter_next (&iter, &key, &value)){
		data[i++] = GPOINTER_TO_INT(key);
	 }
	if (i != size){
		*val = -1;
		free(data);
		return NULL;
	 }
	*val = size;
	return data;
}

int *get_sum(int *size){
	int s,ret = 0;
	int *data = (int *)malloc(topSize*sizeof(int));

	for(int i=0;i < topSize;i++){
		data[i] = 0;
		if (lip[i] != NULL ) {	
			s = (int)g_hash_table_size(lip[i]);
			if (s >= 1){
				data[i] = s;
				ret = 1;
			 } 
		} 
	}

	if (ret ==1){
		*size = topSize;
		return data;
	} else {
		*size = 0;
		free(data);
		return NULL;
	}
}

int load_rdb(){
	FILE *fp = fopen(dbfilename,"rb");
   if (fp == NULL){
		writelog(LOG_ERR,"Error:open file(%s):",dbfilename);
		return -1;
	}
	int ret,max;
	int Size[topSize];
	int *data;

	ret = fread((void *)&max,sizeof(int),1,fp);
	if (ret != 1) {
		writelog(LOG_ERR,"Error:read 1 open file(%s)",dbfilename);
		fclose(fp);
		return -1;
	 }
		
	ret = fread((void *)Size,sizeof(int),topSize,fp);
	if (ret != topSize) {
		fclose(fp);
		writelog(LOG_ERR,"Error:read (%d) open file(%s) read:",topSize,dbfilename);
		return -1;
	 }
	data = (int *)malloc(max*sizeof(int));
	if (data == NULL){
		free(data);
		fclose(fp);
		writelog(LOG_ERR,"Error:malloc file(%s)",dbfilename);
		return -1;
	 }
	
	for(uint32_t i=0;i < topSize;i++){
		if (Size[i] <= 0)
			continue;
		if(feof(fp) != 0){
			ret = -2; break;
		 }
		ret = fread((void *)data,sizeof(int),Size[i],fp);
		if (ret == Size[i]){
			for(int j=0;j < Size[i];j++)
				add_db(i << 24 | data[j],0);
	  	 } else {
			writelog(LOG_ERR,"Error:read (%d) file(%s)",i,dbfilename);
			ret = -3; break;
		 }
	 }

	free(data);
	fclose(fp);
	if (ret < 0)
	  return Check_Fail;
	return Check_OK;
}

int save_rdb(){
	FILE *fp = fopen(dbfilename,"wb");
   if (fp == NULL){
		writelog(LOG_ERR,"Error:save open file (%s)",dbfilename);
		return -1;
	}
	int Size[topSize];
	int max = 0;
	for(int i=0;i < topSize;i++){
		if (lip[i] == NULL || g_hash_table_size(lip[i]) == 0){ 
			Size[i] = 0;
		} else {	
			Size[i] = (int)g_hash_table_size(lip[i]);
			if (Size[i] > max)
				max = Size[i];
	    }
 	 }
	if (max == 0){
		fclose(fp);
		return -2;
	 }

	int *data = (int *)malloc(max*sizeof(int));
	if (data == NULL){
		fclose(fp);
		unlink(dbfilename);
		writelog(LOG_ERR,"Error:save file(%s):malloc",dbfilename);
		return -3;
	 }

	int ret = fwrite((void *)&max,sizeof(int),1,fp);
	if (ret != 1){
		fclose(fp);
		free(data);
		unlink(dbfilename);
		writelog(LOG_ERR,"Error:save file(%s):write data",dbfilename);
		return -4;
	 }
	ret = fwrite((void *)Size,sizeof(int),topSize,fp);
	if (ret != topSize){
		fclose(fp);
		free(data);
		unlink(dbfilename);
		writelog(LOG_ERR,"Error:save file(%s):write data",dbfilename);
		return -5;
	 }

	GHashTableIter iter;
	gpointer key, value;
	int j;
	for(int i=0;i < topSize;i++){
		if (Size[i] == 0) 
			continue;
	
		j = 0;
		g_hash_table_iter_init (&iter, lip[i]);
 		while (g_hash_table_iter_next(&iter, &key, &value))
			data[j++] = GPOINTER_TO_INT(key);

		if (j != Size[i]){
			ret = -6;
			break;
		}  
		ret = fwrite((void *)data,sizeof(int),Size[i],fp);
		if (ret != Size[i]){
			writelog(LOG_ERR,"Error:save file(%s):write data",dbfilename);
			ret = -7;
			break;
		 }
	 }

	fclose(fp);
	free(data);
	if (ret <= 0){
	  	unlink(dbfilename);
		return Check_Fail;
	}
	Changed = 0;
	return Check_OK;
}

int open_db(int flag,char *filename){
	int ret = Check_OK;
	clean_db();
	if (filename != NULL)
		strcpy(dbfilename,filename);

	if (flag == 1 && filename != NULL)
	  	ret = load_rdb();
	return ret;
}

int close_db(int flag){
 /*	int ret = Check_OK;
	if (flag == 1 && Changed == 1)
		ret = save_rdb(); */
	destroy_db();
	return Check_OK;
}

int check_val3(int flag,int size,ipaddr_range_t *rr){
 	if (flag == BLK_ADD) {
		for (int i=0; i < size ; i++)
			add_db(rr[i].num,rr[i].range);

    } else if (flag == BLK_DEL) {
		for (int i=0; i < size ; i++)
			del_db(rr[i].num,rr[i].range);
	 }
	return Check_OK;
}

char * check_val2(int flag,int top,int *size){

 	if (flag == BLK_LIST) 
		return (char *)get_datalist(top,size);
	else		
		return (char *)get_sum(size);
}

int check_val(int flag,uint32_t num,int range){

	switch (flag){
		case BLK_CHECK:
			return check_db(num,range);
			break;
     	case BLK_ADD: 
			add_db(num,range);
			clean_nfcache(BLK_ADD);
			return Check_OK;
        	break;

		case BLK_CHkADD:
			if (check_db(num,range) == BlackOFF){
				clean_nfcache(BLK_CHkADD);
				add_db(num,range);
				return Check_OK;
			 }
			return BlackON;
			break;
		case BLK_DEL: 
			clean_nfcache(BLK_DEL);
			return del_db(num,range);
			break;
 
 		case BLK_SAVE: 
 			return save_rdb();
			break;
		case BLK_LOAD: 
 			return load_rdb();
			break;
 		case BLK_CLEAN:
			clean_nfcache(BLK_CLEAN);
 			clear_db();
			return Check_OK;
			break;
	 }
	return Check_Fail;
}


