#!/bin/bash
#
#	iptips.sh
#		2013.10.25
#		By H.Nakano
#
export PATH=/usr/local/sbin:/usr/local/bin:.:/sbin:/usr/sbin:/bin:/usr/bin

if [ "$EUID" != "0" ]; then 
    echo "must be super-user."
    exit 2
fi

## modify
##	 conf/blackdb.conf, nfqueue.conf, klogview.conf
##	 Host,Passwd

Target="/var/lib/iptips"
BlackDaemon=1
blackdb="$Target/blackipd"
#nfqd="$Target/nfqd"
Conf="$Target/conf/blackdb.conf"
NConf="$Target/conf/nfqueue.conf"
Lockfile="/var/lock/subsys/blackipd"

DistDir="/usr/local/bin"
blackip="$DistDir/blackip"
## parameter
Host="127.0.0.1"
Passwd=1234567

## first: run mk_nfque.lua
start_nfque="$Target/start_nfque.sh"
stop_nfque="$Target/stop_nfque.lua"

if [ ! -x "$blackdb" ]; then
	echo "Couldn't be found a program($blackdb)."
	exit 3
fi

if [ ! -x "$blackip" ]; then
	echo "Couldn't be found a program($blackip)."
	exit 3
fi

if [ ! -f "$Conf" ]; then
	echo "couldn't be found config($Conf)."
	exit 4
fi

if [ ! -x "$start_nfque" ]; then
	echo "couldn't be found script($start_nfque)."
	echo "First: run mk_nfque.lua ../conf/nfqueue.conf"
	exit 5
fi

function start_blackip() {
	if [ "$BlackDaemon" -eq 1 ]; then
   		$blackdb -d -c $Conf
	else
		$blackdb -c $Conf
	fi
   RETVAL=$?
	if [ $RETVAL -eq 0 ]; then
		$start_nfque
		touch "$Lockfile"

		# $nfqd -d -c $NConf
	fi
	return $RETVAL
}

function stop_blackip() {
	$stop_nfque

    $blackip -r $Host -p $Passwd -k
	 RETVAL=$?
	 [ $RETVAL -eq 0 ] && rm -f "$Lockfile"
	return $RETVAL
}

#
case "$1" in
	clean)
		$stop_nfque
		RETVAL=0
		;;
 	start)
		if [ -f "$Lockfile" ]; then
			echo "blackipd is running."
			exit 1
		fi

		start_blackip 
		;;
  	stop)
		if [ ! -f "$Lockfile" ]; then
			exit 1
		fi

		stop_blackip
		;;

	restart)
		if [ -f "$Lockfile" ]; then
			$start_nfque
		else
		 	start_blackip 
		fi
		;;
  	*)
		echo "Usage: $0 {start|stop|restart|clean}"
		exit 1
esac

exit  $RETVAL

