#!/usr/bin/env lua
--
--	mk_nfque.lua
--		2013.10.11
--		By H.Nakano
--
-- # --queue-bypass       
-- # kernel >= 2.6.39 and iptables 1.4.11
--
-- # --queue-balance 
-- # kernel >= 2.6.31 and iptables 1.4.5

require("strict")
local config = require("pl.config")
local Date = require("pl.Date")

local filename = "start_nfque.sh"

local Header = [[#!/bin/bash
#  setting script iptables NFQUEUE from nfqueue.conf.
#    2013.10.11
#    By H.Nakano
#  created %s
#
export PATH=/usr/local/sbin:/usr/local/bin:/sbin:/usr/sbin:/bin:/usr/bin

if [ "$EUID" != "0" ]; then 
    echo "must be super-user."
    exit 2
fi

]]

local Version = 0.1
local Usage = string.format([[
Usage: mk_nfque.lua  conf_file
  ex)
    mk_nfque.lua ../conf/nfqueue.conf

    version %s
      2013.10.11
     By H.Nakano
]],Version)

local function usage()
    print(Usage)
    os.exit()
end

local Parm

local function set_nfqueue(name,val)
	local dat

	if val.num == nil then
		print("error: nfqueue number is NULL")
		return nil
	end

	if val.filter == "raw" then
		dat = "iptables -t raw -I PREROUTING" 
		if val.eth ~= nil then
			dat = dat .. " -i " .. val.eth
		end
	elseif val.filter == "mangle" then
		dat = "iptables -t mangle -I POSTROUTING" 
		if val.eth ~= nil then
			dat = dat .. " -o " .. val.eth
		end
	else
		print("error: filter type",val.filter)
		return nil
	end

	if val.proto ~= nil then
		dat = dat .. " -p " .. val.proto
	end

	if type(val.num) == "string" then
		dat = dat .. " -j NFQUEUE --queue-bypass --queue-balance " .. val.num
	else 
		dat = dat .. " -j NFQUEUE --queue-bypass --queue-num " .. tostring(val.num)
	end
	return dat
end

-- ### ---
if arg[1] == nil then
    usage()
else

	Parm = config.read(arg[1])
	if Parm == nil then 
		error_print("Could'nt be found config:" .. arg[1])
	end

	local val = {}
	for n,v in pairs(Parm) do
		if n ~= "global" then
			dat = set_nfqueue(n,v)
			if dat == nil then
				break
			else
				table.insert(val,dat)
			end
		end
	end

	local fp,err = io.open(filename,"w")
	if fp == nil then
        print(string.format("Error: file open (%s)",filename) ,err)
   		 os.exit()
	end

	fp:write(string.format(Header,tostring(Date())))
	table.sort(val)
	for _,v in ipairs(val) do
		fp:write(v,"\n")
	end
	
	io.close(fp)
	os.execute(string.format("chmod 755 %s",filename))
end


