/*********************** 
	msgC.c	client

		write 

**************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <time.h>
#include <fcntl.h> 
#include <sys/stat.h>
#include <syslog.h>
#include <mqueue.h>
#include <errno.h>

extern ssize_t mq_timedreceive(mqd_t mqdes, char *msg_ptr,
            size_t msg_len, unsigned *msg_prio, const struct timespec *abs_timeout);

#include "ipt_msg.h"
#include "hashdb.h"

extern void writelog(int prio,char *fmt, ...);

static	mqd_t mqd_r,mqd_w;
static unsigned int prio = MQ_Priority;
static 	char *rbuf;
static struct mq_attr attr;

void close_msg_client(){
	free(rbuf);
	mq_close(mqd_r);
	mq_close(mqd_w);
}

/* 9*60 */
struct timezone tz = {-540,0};

int init_msg_client()
{
	errno = 0;
	mqd_r = mq_open(MQ_WRITE1,O_RDONLY); 
    if (mqd_r == (mqd_t)-1){
		 writelog(LOG_ERR,"Error:msgC (mq_open) read (%s)",strerror(errno));
		 return -1;
	 }
	mqd_w = mq_open(MQ_READ1 ,O_WRONLY);
    if (mqd_w == (mqd_t)-1){
 		writelog(LOG_ERR,"Error:msgC (mq_open) write (%s)",strerror(errno));
		mq_close(mqd_r);
		return -1;
	 }

	mq_getattr(mqd_r,&attr);
	rbuf = malloc(attr.mq_msgsize);
	if(rbuf == NULL) {
		mq_close(mqd_r);
		mq_close(mqd_w);
		writelog(LOG_ERR,"Error:msgC malloc");
		return -1;
	}
	return 0;
}

int check_blackip(uint32_t qnum,uint32_t addr){
	struct timeval now;
	struct timespec ts;
	arg_buff_t *rr;

		memset((void *)rbuf,0,attr.mq_msgsize);
		rr = (arg_buff_t *)rbuf;
		rr->status = REQUEST_ADDR;
		rr->flag = BLK_CHECK;
		rr->qnum = qnum;
		rr->num = addr;
		rr->range = 0;
	//	printf("write...\n");
		errno = 0;
		if (mq_send(mqd_w,rbuf,attr.mq_msgsize, prio) != 0) {
			writelog(LOG_ERR,"Error:msgC mq_send (%s)",strerror(errno));
			return BlackOFF;
		}
	
	//	printf("read...\n");
		gettimeofday(&now,&tz);
		long tt = (now.tv_usec + MQ_Wait) * 1000;
		if (tt >= NanoSec){
  			ts.tv_sec  = now.tv_sec + 1;
			ts.tv_nsec = tt - NanoSec;
		 } else {
   			ts.tv_sec  = now.tv_sec;
			ts.tv_nsec = tt;
		 }

	 	int n = mq_timedreceive(mqd_r, rbuf, attr.mq_msgsize, &prio, &ts);
		if (n > 0 ) {
			rr = (arg_buff_t *)rbuf;
			// printf("status: %d , qnum: %d , addr:%d\n",rr->status,rr->flag,rr->num);
			return rr->status;  // BlackON == 1
		} 
		return BlackOFF;
}

