/********************
	msgS.c
		2013.8.30
		By H.Nakano

 	POSIX msg queue 
**********************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h> 
#include <sys/stat.h> 
#include <stdbool.h>
#include <stdint.h>
#include <time.h>
#include <mqueue.h>
#include <syslog.h>
#include <errno.h>

extern ssize_t mq_timedreceive(mqd_t mqdes, char *msg_ptr,
            size_t msg_len, unsigned *msg_prio, const struct timespec *abs_timeout);

#include "ipt_msg.h"
#include "hashdb.h"
/* hashdb.c */
extern int check_db(uint32_t num,int range);
extern void writelog(int prio,char *fmt, ...);

bool quit = false;

void stop_msg(){
	quit = true;
}

int init_msg_server(){
	unsigned int prio = MQ_Priority;
	arg_buff_t *rr;
 	mqd_t mqd_r,mqd_w;
	struct mq_attr attr;
	struct timespec ts;
	char *buf;

	mq_unlink(MQ_READ1);
	mq_unlink(MQ_WRITE1);

	uid_t uid = getuid();
	if (uid == 0){
		attr.mq_maxmsg = Msg_Max;
		attr.mq_msgsize = Msg_Size;
	  	mqd_r = mq_open(MQ_READ1,O_RDONLY|O_CREAT,MODE_File,&attr);
	} else 
    	mqd_r = mq_open(MQ_READ1,O_RDONLY|O_CREAT,MODE_File,NULL);
	
	errno = 0;
    if (mqd_r == (mqd_t) -1){
		 writelog(LOG_ERR,"Error:msgS (mq_open) read (%s)",strerror(errno));
		 return -1;
	 }
	if (uid == 0){
		attr.mq_maxmsg = Msg_Max;
		attr.mq_msgsize = Msg_Size;
		mqd_w = mq_open(MQ_WRITE1,O_WRONLY|O_CREAT,MODE_File,&attr);
	} else
   		mqd_w = mq_open(MQ_WRITE1,O_WRONLY|O_CREAT,MODE_File,NULL);

	errno = 0;
    if (mqd_w == (mqd_t) -1){
		 writelog(LOG_ERR,"Error:msgS (mq_open) write (%s)",strerror(errno));
		 mq_close(mqd_r);
		 return -1;
	 }

	mq_getattr(mqd_r,&attr);
	buf = malloc(attr.mq_msgsize);
	if(buf == NULL) {
		mq_close(mqd_r);
		mq_close(mqd_w);
		writelog(LOG_ERR,"Error: msgS malloc");
		return -1;
	}

//	printf("buf size:%ld ,queue:%ld\n",(long)attr.mq_msgsize,(long)attr.mq_maxmsg);
	ts.tv_nsec = 0;

 	while (!quit){
		memset((void *)buf,0,attr.mq_msgsize);
		ts.tv_sec = time(NULL) + 2;
	 	int n = mq_timedreceive(mqd_r, buf, attr.mq_msgsize, &prio, &ts);
	 	if (n > 1) {
			rr = (arg_buff_t *)buf;
			rr->status = check_db(rr->num,0);
			errno = 0;
	   		if (mq_send(mqd_w, buf,attr.mq_msgsize, prio) != 0){ 
				writelog(LOG_ERR,"Error: msgS mq_send (%d:%u) (%s)",rr->qnum,rr->num,strerror(errno));
			 }
     	 } 
 	}

 	mq_close(mqd_r);
	mq_close(mqd_w);
	free(buf);
  	return 0;
}


