/************************
  	nfqd.c
		2013.8.30
		By H.Nakano

		version 0.5
--		email: hnakano0157@gmail.com
--		copyright H.Nakano
--		License GPL version 3
****************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <getopt.h>
#include <syslog.h>
#include <errno.h>

#include <glib.h>

#include "ipt_msg.h"
#include "nfqueue.h"

/* nfqueue.c */
extern int nf_open(global_parm_t *gparm,nfque_parm_t *parm);

/* check_port.c */
extern int get_tokennum(char *tokens);

/* write_log.c */
extern void set_daemon();
extern void open_log(char *keyname,int facility);
extern void open_zlog(char *filename,char *keys);
extern void close_log();
extern void writelog(int prio,char *fmt, ...);

void usage(){
	printf("usage\n");
	printf("\t must be a super-user mode.\n");
	printf("\t nfqd option -c confg-file \n");
	printf("\t -d,--daemon  (default no-daemon)\n");
	printf("\t -c,--config  config-file \n");
	
	printf("\tex)\n");
	printf("\t nfqd -d -c ../conf/nfqueue.conf\n");
	printf("\t nfqd -c ../conf/nfqueue.conf\n\n");
	printf("\t Version 0.5\n");
	exit(1);
}

nfque_parm_t * get_config(global_parm_t *gparm,char *filename){
	int  i,num,pp = 0;
	int  nfq;
	gsize queuemax;
	nfque_parm_t *parm;
	GKeyFile *gfile;
  	gint start,end,err;
	gchar *val,**tokens;
	gchar *proto,*eth,*filter;
	gchar **groups,*group;

	memset((void *)gparm,0,sizeof(global_parm_t));

  	gfile = g_key_file_new();  
  	if (g_key_file_load_from_file(gfile,filename, 0,NULL) == FALSE){
		writelog(LOG_ERR,"Error: read file(%s)", filename);
		return NULL;
	}
	g_key_file_set_list_separator(gfile,',');

	val = g_key_file_get_value(gfile,"global","zlog",NULL);
	if (val != NULL && g_file_test(val,G_FILE_TEST_IS_REGULAR) == TRUE)
		strcpy(gparm->zlogconf,val);
	if (val != NULL)
		g_free(val);

	val = g_key_file_get_value(gfile,"global","drop",NULL);
	if (val != NULL && g_file_test(val,G_FILE_TEST_IS_REGULAR) == TRUE)
		strcpy(gparm->dropconf,val);
	if (val != NULL)
		g_free(val);

	pp = g_key_file_get_integer(gfile,"global","print_level",NULL);
	gparm->printLevel = pp;
	groups = g_key_file_get_groups(gfile,&queuemax);
	if (queuemax <= 1){
		if (groups != NULL)
			g_strfreev(groups);
		g_key_file_free(gfile);
		writelog(LOG_ERR,"Error: groups");
		return NULL;
	 }
	nfq = queuemax -1;
	parm = (nfque_parm_t *)calloc(nfq,sizeof(nfque_parm_t));
	num =0;	
	for(i=0;i < queuemax;i++){
		group = groups[i];
		if (strcmp(group,"global") == 0)
			continue;
		if (g_key_file_has_group(gfile,group) == FALSE){
			writelog(LOG_ERR,"Error: group (%s)",group);
			continue;
		}
	
		val = g_key_file_get_value(gfile,group,"num",NULL);
		if (val == NULL){
			g_key_file_free(gfile);
			writelog(LOG_ERR,"Error:(%s) queue_num is NULL",group);
			num = -1;
			break;
		 }
		end = 0;
		err = 0;
	 	tokens = g_strsplit(val,":",2);
		if (tokens[1]){
	   		start = get_tokennum(tokens[0]);
			end   = get_tokennum(tokens[1]);
			if (start < 0 || end < 0 || start >= end)
				err =1;
		} else {
			end = 0;
			start = get_tokennum(tokens[0]);
			if (start < 0)
				err =1;
	    }
		g_strfreev(tokens);

	//	writelog(LOG_INFO,"start:%d:%d",start,end);

		if (err){
			writelog(LOG_ERR,"Error:(%s) queue_num(%s)",group,val);
			g_free(val);
			num = -1;
			break;
		 }
		g_free(val);

		filter = g_key_file_get_string(gfile,group,"filter",NULL);
		if (filter){
			strcpy(parm[num].filter, g_ascii_strdown(filter,strlen(filter))); 
		 } else {
			g_free(filter);
			num = -1;
			break;
		}

		parm[num].queue_num = start;
		parm[num].queue_end = end; 
		proto = g_key_file_get_string(gfile,group,"proto",NULL);
		eth   = g_key_file_get_string(gfile,group,"eth",NULL);
		if (proto){
			strcpy(parm[num].proto, g_ascii_strdown(proto,strlen(proto))); 
			g_free(proto);
		 }
		if (eth){
			strcpy(parm[num].eth, g_ascii_strdown(eth,strlen(eth)));
			g_free(eth);
		 }
	
		if (parm[num].queue_end > 0){
			int delt = end - start;
			parm = (nfque_parm_t *)realloc(parm,(delt + nfq)*sizeof(nfque_parm_t));
			for( int j=1; j <= delt; j++){
				memcpy((void *)&parm[num+j],(void *)&parm[num],sizeof(nfque_parm_t));
				parm[num + j].queue_num = start +j;
			 }
			nfq += delt;
			num += delt;
		 }
		num ++;
	}

	g_strfreev(groups);
	g_key_file_free(gfile);
	gparm->rule_num = num;
	if(num <= 0)
		return NULL;
	return parm;
}

int nfq_fork(global_parm_t *gparm,nfque_parm_t *parm){

	for (int i =0;i < gparm->rule_num;i++){
		errno = 0;
		pid_t child = fork();
	  	if (child == 0) {
			int cc = 0;
			while (cc < 4){
				int ret = nf_open(gparm,&parm[i]);
				if (ret == 0) 
					break;
				else {
					usleep(10);
					cc ++;
			 	} 
			} 
		/*	int ret = nf_open(gparm,&parm[i]);
			if (ret != 0)
				writelog(LOG_ERR, "Error: Nfqd start(%d) (%d)",i,ret);  */
			exit(0);
		} else if (child == -1) {
			writelog(LOG_ERR, "Error: Nfqd fork (%s)",strerror(errno));
			exit(1);
	    }		  
  	}
	return 0;
}

int main(int argc,char **argv){
  	int c;
  	int dflag = 0;
	char *filename = NULL;
	global_parm_t gparm;
	nfque_parm_t *parm;

	const struct option longopt[] = {
    	{"help", no_argument, NULL, '?'},
    	{"daemon", no_argument, NULL, 'd'},
    	{"config", required_argument, NULL, 'c'},
    	{NULL, 0, NULL, 0}
     };

  	opterr = 0;
 	while ((c = getopt_long(argc, argv, "dc:",longopt,NULL)) != -1){
     switch (c)
       {
      	case 'd': // daemon
        	dflag = 1;
        	break;
      	case 'c': // config
        	filename = optarg;
        	break;
      	default:
        	usage();
			break;
       }
	}

	if (filename == NULL){
		writelog(LOG_ERR, "Error: Nfqd No config-file");
		usage();
		return -1;
	 } 

	parm = get_config(&gparm,filename);
	if (parm == NULL){
		writelog(LOG_ERR, "Error: Nfqd config-file (%s)",filename);
		return -1;
	 } 

	uid_t uid = getuid();
	if (uid != 0){
		free(parm);
		writelog(LOG_INFO,"Error: Nfqd must be a super-user.");
		return -1;
	 }

	if (dflag == 1){
		if(daemon(0,0) == -1){
			writelog(LOG_ERR,"Error: Nfqd start daemon.");
			free(parm);
			close_log();
      		return -1;
		 }
	}
	sleep(1);
	nfq_fork(&gparm,parm);
	sleep(8);
	free(parm);
	return 0;
}


