/************************************
	nfqueue.c
		2013.8.10
		By H.Nakano
	version 0.1
*************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <errno.h>
#include <stdint.h>
#include <signal.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <sys/time.h>
#include <sys/resource.h>

#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/icmp.h>

#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <libnfnetlink/libnfnetlink.h>
#include <libnetfilter_queue/libnetfilter_queue.h>

#include "hashdb.h"
#include "nfqueue.h"

#define NFQ_BUFSIZE (131072)
// (262144)

/* msgC.c */
extern int  init_msg_client();
extern void close_msg_client();
extern int  check_blackip(int sem_num,uint32_t data);

/* addr_tools.c */
extern int addr2num(char *dat,uint32_t *val,int *range);

/* drop_rule.c */
extern uint32_t check_dpport(drop_rule_t *d,ipt_info_t *ipt_info);
extern void     set_dparm(int num,nfque_parm_t *parm,drop_rule_t *dparm);
extern drop_rule_t *get_droprule(char *filename,int *num);
extern void d_free(int num,drop_rule_t *dparm);

/* getipaddr.c */
extern void free_hinfo(devip4addr_t *addr);
extern int  get_devip4addr(devip4addr_t **ip4addr);
extern void print_hostinfo(devip4addr_t *addr);

/* print_pkt.c */
extern void print_header(int plevel,ipt_info_t *ipt_info);

/* check_ksupport.c */
extern int check_ksupport(int m,int s,int u);
/* write_log.c */
extern int  open_zlog(char *filename,char *key);
extern void close_log();
extern void writelog(int prio,char *fmt, ...);

static nfdef_t nfq = {0};

void get_ipv4(unsigned char *data,ipt_info_t *ipt_info){
	struct iphdr ipv4h;
	uint16_t offset;
	char *tmp;

	memcpy(&ipv4h,data,sizeof(struct iphdr));
	tmp = inet_ntoa((struct in_addr){ipv4h.saddr});
	strcpy(ipt_info->srcip,tmp);
	tmp = inet_ntoa((struct in_addr){ipv4h.daddr});
	strcpy(ipt_info->dstip,tmp);
	
	ipt_info->protocol = ipv4h.protocol;
	ipt_info->pktlen   = ntohs(ipv4h.tot_len);
	offset = sizeof(struct iphdr);

	if (ipv4h.protocol == 6) {
		struct tcphdr tcph;
		memcpy(&tcph,data + offset,sizeof(struct tcphdr));
		ipt_info->sport = ntohs(tcph.source);
		ipt_info->dport = ntohs(tcph.dest);
		ipt_info->tcp_window = ntohs(tcph.window); 
		ipt_info->cwr  = tcph.cwr;
		ipt_info->ece  = tcph.ece;
   	 	ipt_info->urg  = tcph.urg;
    	ipt_info->ack  = tcph.ack;
    	ipt_info->psh  = tcph.psh;
    	ipt_info->rst  = tcph.rst;
    	ipt_info->syn  = tcph.syn;
    	ipt_info->fin  = tcph.fin;

		offset += tcph.doff;

	} else if(ipv4h.protocol == 17) {
		struct udphdr udph;
		memcpy(&udph,data + offset,sizeof(struct udphdr));
		ipt_info->sport = ntohs(udph.source);
		ipt_info->dport = ntohs(udph.dest);
		offset += sizeof(struct udphdr);

	} else if (ipv4h.protocol == 1) {
		struct icmphdr icmph;
		memcpy(&icmph,data + offset,4);
		ipt_info->icmp_type = icmph.type;
		ipt_info->icmp_code = icmph.code;
		offset += 4;
	}
	ipt_info->offset = offset;
}

void check_dev(struct nfq_data *tb,ipt_info_t *ipt_info)
{
	struct nlif_handle *nlif;
	struct nfqnl_msg_packet_hw *hwph = nfq_get_packet_hw(tb);

		if (hwph) {
			char *mac = ipt_info->mac;
			int hlen = ntohs(hwph->hw_addrlen);
			for (int i = 0; i < hlen; i++){
			  snprintf(mac + i*3,4,"%02x:", hwph->hw_addr[i]);
			 }
			mac[17] = '\0';
		}

		nlif = nlif_open();
		if (nlif ){
			uint32_t ifi; 
			nlif_query(nlif);
			ifi = nfq_get_indev(tb);
			if (ifi ) {
				nlif_index2name(nlif, ifi,ipt_info->inth);
		 	}

			ifi = nfq_get_outdev(tb);
			if (ifi) {
				nlif_index2name(nlif, ifi,ipt_info->outth);
		 	}

	/*		ifi = nfq_get_physindev(tb);
			if (ifi) {
				nlif_index2name(nlif, ifi,ipt_info->indev);
		 	}

			ifi = nfq_get_physoutdev(tb);
			if (ifi) {
				nlif_index2name(nlif, ifi,ipt_info->outdev);
		 	} */
			nlif_close(nlif);
		}
}

/************************
 linux/netfilter.h
#define NF_DROP 0
#define NF_ACCEPT 1
#define NF_STOLEN 2
#define NF_QUEUE 3
#define NF_REPEAT 4
#define NF_STOP 5
************************/

uint32_t check_verdict(ipt_info_t *ipt_info){
//	uint32_t verdict = NF_ACCEPT; 
	char *ipaddr;

	if (ipt_info->hook == NF_IP_PRE_ROUTING || ipt_info->hook == NF_IP_LOCAL_IN){
		if (strcmp(ipt_info->inth,"lo") == 0)
			return NF_ACCEPT; 
		ipaddr = ipt_info->srcip;
		ipt_info->reason = B_in;
	 }
	else if (ipt_info->hook == NF_IP_POST_ROUTING || ipt_info->hook == NF_IP_LOCAL_OUT){
		if (strcmp(ipt_info->outth,"lo") == 0)
			return NF_ACCEPT; 
		ipaddr = ipt_info->dstip;
		ipt_info->reason = B_out;
	 }
 	else if (ipt_info->hook == NF_IP_FORWARD){
		if(strcmp(nfq.n->filter,"raw") == 0 || strcmp(nfq.n->filter,"input") == 0){
			ipaddr = ipt_info->srcip;
			ipt_info->reason = B_in;
		} else{
			ipaddr = ipt_info->dstip;
			ipt_info->reason = B_out;
		 }
	 } else {
		ipt_info->reason = B_err;
		return NF_DROP;
	 } 

	if (strcmp(ipaddr,"127.0.0.1") == 0)
		return NF_ACCEPT;

	int range;
	uint32_t num;
	if( addr2num(ipaddr, &num,&range) < 0){
		ipt_info->reason = B_err;
		return NF_DROP;
	 }
	if (check_blackip(nfq.n->queue_num,num) == BlackON)
		return NF_DROP;
	else 
		return NF_ACCEPT;
}

uint32_t check_pkt(struct nfq_data *tb,uint32_t hook){
	ipt_info_t ipt_info = {0};
	uint32_t verdict;
	int size;
	unsigned char *payload;

	ipt_info.hook = hook;
	check_dev(tb,&ipt_info);
	size = nfq_get_payload(tb, &payload);
	if (size < 0) {	
		writelog(LOG_INFO,"Drop Size");
		return NF_DROP; 
	 }

	get_ipv4(payload,&ipt_info);
	verdict = check_verdict(&ipt_info);

	if (verdict != NF_DROP && nfq.d != NULL){
		for(int i=0; i < nfq.dropnum;i++){
			if (nfq.d[i].rulenum >= 0){
		  		verdict = check_dpport(&nfq.d[i],&ipt_info);
				if (verdict == NF_DROP)
					break;
			 }
		}
	 }

	if (verdict == NF_DROP && nfq.printlevel == 1 ){
		if (ipt_info.reason == B_in)
			writelog(LOG_INFO,"DROP %s",ipt_info.srcip);
		else if (ipt_info.reason == B_out)
			writelog(LOG_INFO,"DROP %s",ipt_info.dstip);
		else if (ipt_info.reason == B_cache)
			writelog(LOG_INFO,"DROP Cache");
		else if (ipt_info.reason == B_err)
			writelog(LOG_INFO,"DROP Error from=%s to=%s",ipt_info.srcip,ipt_info.dstip);
		else {
			if (ipt_info.rulename[0] == '\0')
				writelog(LOG_INFO,"DROP (num error) from=%s to=%s",ipt_info.srcip,ipt_info.dstip);
			else
				writelog(LOG_INFO,"DROP (%s) from=%s to=%s",ipt_info.rulename,ipt_info.srcip,ipt_info.dstip);
		 }
		return NF_DROP;
	 }

	/*	if (size > ipt_info.offset){
			verdict = check_payload(size,&ipt_info,payload);
			if (verdict == NF_DROP){
				writelog(LOG_INFO,"Drop payload");
				return NF_DROP;
		     }
		}
	*/ 
	
	if (nfq.printlevel >= 2){
		if (nfq.printlevel == 2 && verdict != NF_DROP)
			return verdict;
		ipt_info.verdict = verdict;
		print_header(nfq.printlevel,&ipt_info);
	}
	return verdict;
}

int queue_callback(struct nfq_q_handle *qh, struct nfgenmsg *nfmsg, struct nfq_data *nfa, void *data){
	struct nfqnl_msg_packet_hdr *ph = nfq_get_msg_packet_hdr(nfa);
	if (ph) {
		uint32_t verdict;
		int ret;
		uint32_t id = ntohl(ph->packet_id);

		if (ntohs(ph->hw_protocol) == 0x0800) // 0x08100 VLAN
			verdict = check_pkt(nfa,ph->hook); // ,data);
		else 
			verdict = NF_DROP;
		ret = nfq_set_verdict(qh, id, verdict, 0, NULL);
		return ret;
	 } 
	return  1;
}

static int quit =0;

void sig_exit(int num){
     quit = 1;
     exit(0);
}

int nf_open(global_parm_t *gparm,nfque_parm_t *parm){
  	int  rv,fd;
	int  ret,size;
	char *buf;

	nfq.n = (nfque_parm_t *)malloc(sizeof(nfque_parm_t));
	memcpy((void *)nfq.n,(void *) parm,sizeof(nfque_parm_t));

	nfq.printlevel = gparm->printLevel;
	if (gparm->zlogconf[0] != '\0'){
		char pname[16];
		sprintf(pname,"nfqd_%d",parm->queue_num);
		if (open_zlog(gparm->zlogconf,pname) < 0)
			writelog(LOG_ERR,"Error: nfqueue zlogconf(%s)",gparm->zlogconf);
	    else 
		 	writelog(LOG_INFO, "nfqueue start (%d)",nfq.n->queue_num);
	}

	/* host info : getipaddr.c */
	if (get_devip4addr(&nfq.host) < 0)
		nfq.host = NULL;
/*	else 
		print_hostinfo(nfq.host);
*/
	/* drop ports: drop_rule.c */
	if (gparm->dropconf[0] != '\0') {
		int dropnum;
		nfq.d = get_droprule(gparm->dropconf,&dropnum);
		if (nfq.d != NULL){
			set_dparm(dropnum,nfq.n,nfq.d);
			nfq.dropnum = dropnum;
		} 
	}

	nfq.h = nfq_open();
	if (! nfq.h) {
		writelog(LOG_ERR,"Error:nfqueue during nfq_open()");
		if (nfq.d != NULL)
			free(nfq.d);
		if (nfq.host != NULL)
			free_hinfo(nfq.host);
		free(nfq.n);
		close_log();
		return -1;
	}

	if (nfq_unbind_pf(nfq.h, AF_INET) < 0) {
		writelog(LOG_ERR,"Error:nfqueue during nfq_unbind_pf()");
		if (nfq.d != NULL)
			free(nfq.d);
		if (nfq.host != NULL)
			free_hinfo(nfq.host);
		nfq_close(nfq.h);
		free(nfq.n);
		close_log();
		return -1;
	}

	if (nfq_bind_pf(nfq.h, AF_INET) < 0) {
		writelog(LOG_ERR,"Error:nfqueue during nfq_bind_pf()");
		if (nfq.d != NULL)
			free(nfq.d);
		if (nfq.host != NULL)
			free_hinfo(nfq.host);
		nfq_close(nfq.h);
		free(nfq.n);
		close_log();
		return -1;
	}

	nfq.qh = nfq_create_queue(nfq.h, nfq.n->queue_num, &queue_callback, NULL);
	if (! nfq.qh) {
		writelog(LOG_ERR,"Error:nfqueue during nfq_create_queue()");
		if (nfq.d != NULL)
			free(nfq.d);
		if (nfq.host != NULL)
			free_hinfo(nfq.host);
		nfq_close(nfq.h);
		free(nfq.n);
		close_log();
		return -1;
	}

	if (nfq_set_mode(nfq.qh, NFQNL_COPY_PACKET,0xffff) < 0) {
		writelog(LOG_ERR,"Error:nfqueue can't set packet_copy mode");
		if (nfq.d != NULL)
			free(nfq.d);
		if (nfq.host != NULL)
			free_hinfo(nfq.host);
		nfq_close(nfq.h);
		free(nfq.n);
		close_log();
		return -1;
	}

	/* MSG */
	ret = init_msg_client();
	if (ret <= -1){
		if (nfq.d != NULL)
			free(nfq.d);
		if (nfq.host != NULL)
			free_hinfo(nfq.host);
		nfq_destroy_queue(nfq.qh);
		nfq_close(nfq.h);
		free(nfq.n);
		close_log();
	  	return -2;
     }

	signal(10,sig_exit);

	if (strcmp(parm->proto,"tcp") == 0)
		size = -17;
	else
		size = -16;
	setpriority(PRIO_PROCESS,getpid() ,size);
	writelog(LOG_INFO,"nice (%d)",getpriority(PRIO_PROCESS,getpid()));

	fd = nfq_fd(nfq.h);
	/* kernel >= 2.6.30 */
	if (check_ksupport(2,6,30) == 1){
		errno = 0;
		size =1;
		if ( setsockopt(fd, SOL_NETLINK,NETLINK_NO_ENOBUFS, &size, sizeof(int)) < 0)
			writelog(LOG_ERR,"Error:nfqueue setsockopt (%s)",strerror(errno));
		size = NFQ_BUFSIZE;
	} else 
		size = NFQ_BUFSIZE*2;

	errno = 0;
	size = nfnl_rcvbufsiz(nfq_nfnlh(nfq.h),size);
	buf  = (char *)malloc(size*sizeof(char));
	if (buf == NULL){
		size = nfnl_rcvbufsiz(nfq_nfnlh(nfq.h),NFQ_BUFSIZE/2);
		errno = 0;
		buf = (char *)malloc(size*sizeof(char));
		if (buf == NULL){
		  	writelog(LOG_ERR,"Error:malloc recvbufsize (%s)",strerror(errno));
			quit = 1;
		}
	 } 
	if (buf != NULL)
		writelog(LOG_INFO,"recvbufsize (%d)",size);

	while (quit == 0) {
		errno = 0;
		rv = recv(fd, buf,size, 0);
		if (rv >= 0) {
			nfq_handle_packet(nfq.h, buf, rv);
			continue;
		}

		if (errno == ENOBUFS) {
			writelog(LOG_ERR,"Error:nfqueue losing packets (%s)",strerror(errno));
			continue;
		 }
		writelog(LOG_ERR,"Error:nfqueue recv (%s)",strerror(errno));
		break;
	}

	free(buf);
	close_msg_client();
	nfq_destroy_queue(nfq.qh);
	nfq_close(nfq.h);

	if (nfq.d != NULL)
		free(nfq.d);

	if (nfq.host != NULL)
		free_hinfo(nfq.host);

	free(nfq.n);
	writelog(LOG_INFO, "nfqueue end (%d)",nfq.n->queue_num);
	close_log();
	return 0;
}

