/************************************
	parse_name.c
		2013.10.3
		By H.Nakano
	version 0.1

	from)
		iptables/extensions/libipt_icmp.c

*************************************/
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <strings.h>
#include <netinet/in.h> /* protocol */

#include "nfqueue.h"

#define ARRAY_SIZE(x) (sizeof(x) / sizeof(*(x)))

struct icmp_names {
	const char *name;
	uint8_t type;
	uint8_t code_min, code_max;
};

static const struct icmp_names icmp_codes[] = {
	{ "any", 0xFF, 0, 0xFF },
	{ "echo-reply", 0, 0, 0xFF },
	/* Alias */ { "pong", 0, 0, 0xFF },
	{ "destination-unreachable", 3, 0, 0xFF },
	{ "network-unreachable", 3, 0, 0 },
	{ "host-unreachable", 3, 1, 1 },
	{ "protocol-unreachable", 3, 2, 2 },
	{ "port-unreachable", 3, 3, 3 },
	{ "fragmentation-needed", 3, 4, 4 },
	{ "source-route-failed", 3, 5, 5 },
	{ "network-unknown", 3, 6, 6 },
	{ "host-unknown", 3, 7, 7 },
	{ "network-prohibited", 3, 9, 9 },
	{ "host-prohibited", 3, 10, 10 },
	{ "TOS-network-unreachable", 3, 11, 11 },
	{ "TOS-host-unreachable", 3, 12, 12 },
	{ "communication-prohibited", 3, 13, 13 },
	{ "host-precedence-violation", 3, 14, 14 },
	{ "precedence-cutoff", 3, 15, 15 },
	{ "source-quench", 4, 0, 0xFF },
	{ "redirect", 5, 0, 0xFF },
	{ "network-redirect", 5, 0, 0 },
	{ "host-redirect", 5, 1, 1 },
	{ "TOS-network-redirect", 5, 2, 2 },
	{ "TOS-host-redirect", 5, 3, 3 },
	{ "echo-request", 8, 0, 0xFF },
	/* Alias */ { "ping", 8, 0, 0xFF },
	{ "router-advertisement", 9, 0, 0xFF },
	{ "router-solicitation", 10, 0, 0xFF },
	{ "time-exceeded", 11, 0, 0xFF },
	/* Alias */ { "ttl-exceeded", 11, 0, 0xFF },
	{ "ttl-zero-during-transit", 11, 0, 0 },
	{ "ttl-zero-during-reassembly", 11, 1, 1 },
	{ "parameter-problem", 12, 0, 0xFF },
	{ "ip-header-bad", 12, 0, 0 },
	{ "required-option-missing", 12, 1, 1 },
	{ "timestamp-request", 13, 0, 0xFF },
	{ "timestamp-reply", 14, 0, 0xFF },
	{ "address-mask-request", 17, 0, 0xFF },
	{ "address-mask-reply", 18, 0, 0xFF }
};

struct xtables_pprot {
	const char *name;
	uint8_t num;
};

const struct xtables_pprot xtables_chain_protos[] = {
	{"tcp",       IPPROTO_TCP},
	{"sctp",      IPPROTO_SCTP},
	{"udp",       IPPROTO_UDP},
	{"udplite",   IPPROTO_UDPLITE},
	{"icmp",      IPPROTO_ICMP},
//	{"icmpv6",    IPPROTO_ICMPV6},
//	{"ipv6-icmp", IPPROTO_ICMPV6},
	{"esp",       IPPROTO_ESP},
	{"ah",        IPPROTO_AH},
//	{"ipv6-mh",   IPPROTO_MH},
	{"mh",        IPPROTO_MH},
	{"all",       0}
};

uint8_t parse_protocol(const char *name){

	for (int i = 0; i < ARRAY_SIZE(xtables_chain_protos);i++) {
		if (strcmp(name, xtables_chain_protos[i].name) == 0)
			return xtables_chain_protos[i].num;
	 }
	return -1;
}

int parse_icmp(const char *icmptype,icmp_t *code){

	for (int i = 0; i < ARRAY_SIZE(icmp_codes); i++) {
		if (strncasecmp(icmp_codes[i].name, icmptype, strlen(icmptype)) == 0) {
			code->type     = (uint16_t)icmp_codes[i].type;
			code->code_min = (uint16_t)icmp_codes[i].code_min;
			code->code_max = (uint16_t)icmp_codes[i].code_max;
			return 0;
		} 
	}
	return -1;
}



