/************************************
	print_pkt.c
		2013.8.15
		By H.Nakano

*************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <syslog.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <linux/if.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>

#include "nfqueue.h"

extern void writelog(int prio,char *fmt, ...);

char *get_pkt_direction(uint32_t hook){
	switch (hook){
			case NF_IP_PRE_ROUTING:
				return "PreRouting";
				break;
			case NF_IP_LOCAL_IN:
				return "LocalIn";
				break;
			case NF_IP_LOCAL_OUT:
				return "LocalOut";
				break;
			case NF_IP_POST_ROUTING:
				return "PostRouting";
				break;
			case NF_IP_FORWARD:
				return "Forward";
				break;
			default:
				return "NULL";
				break;
	}
}

void get_pct_flags(ipt_info_t *ipt_info,char *tmp){
	int  offset = 0;

	if (ipt_info->cwr)
		{	strcpy(tmp,"CWR "); offset += 4; }
	if (ipt_info->ece)
		{	strcpy(tmp,"ECE "); offset += 4; }
	if (ipt_info->urg)
	 	{	strcpy(tmp,"URG "); offset += 4; }
	if (ipt_info->ack)
	 	{	strcpy(tmp + offset,"ACK "); offset += 4;}
    if (ipt_info->psh)
 		{	strcpy(tmp + offset,"PSH "); offset += 4;}
    if	(ipt_info->rst)
 		{	strcpy(tmp + offset,"RST "); offset += 4;}
    if (ipt_info->syn)
 		{	strcpy(tmp + offset,"SYN "); offset += 4;}
    if (ipt_info->fin)
		 	strcpy(tmp + offset,"FIN");
}

/************************
 linux/netfilter.h
#define NF_DROP 0
#define NF_ACCEPT 1
#define NF_STOLEN 2
#define NF_QUEUE 3
#define NF_REPEAT 4
#define NF_STOP 5
************************/

char * print_vervict(uint32_t verdict){

	switch (verdict){
		case NF_DROP:
			return "DROP"; 
			break;
		case NF_ACCEPT:
			return "ACCEPT"; 
			break;
		case NF_STOLEN:
			return "NF_STOLEN"; 
			break;
		case NF_QUEUE:
			return "NF_QUEUE"; 
			break;
		case NF_REPEAT:
			return "NF_REPEAT"; 
			break;
		case NF_STOP:
			return "STOP"; 
			break;
		default:
			return "NULL"; 
			break;
	}
}

void print_header(int plevel,ipt_info_t *ipt_info){
	char dat[256];
	char *hook = get_pkt_direction(ipt_info->hook);

	if (ipt_info->verdict == NF_DROP){
		if (ipt_info->reason == B_in)
			sprintf(dat,"DROP %s ",ipt_info->srcip);
		else if (ipt_info->reason == B_out)
			sprintf(dat,"DROP %s ",ipt_info->dstip);
		else if(ipt_info->reason == B_cache)
			return; 
		else if (ipt_info->reason == B_err)
			sprintf(dat,"DROP Error");
		else {
			if (ipt_info->rulename[0] == '\0') 
				strcpy(dat,"DROP (num error) ");
			else
				sprintf(dat,"DROP (%s) ", ipt_info->rulename);
		 }

		sprintf(&dat[strlen(dat)], "From=%s To=%s %s ",ipt_info->srcip,ipt_info->dstip,hook);
	 } else {
		sprintf(dat,"%s From=%s To=%s %s ",print_vervict(ipt_info->verdict),ipt_info->srcip,ipt_info->dstip,hook);
	}

	if ( strlen(ipt_info->mac) == 17) 
		sprintf(&dat[strlen(dat)], "MAC=%s ",ipt_info->mac);
	if ( strlen(ipt_info->inth) >= 3)
		sprintf(&dat[strlen(dat)],"IN=%s ",ipt_info->inth);
	if ( strlen(ipt_info->outth) >= 3)
		sprintf(&dat[strlen(dat)],"OUT=%s ",ipt_info->outth);
/*
	if ( strlen(ipt_info->indev) >= 3)
		sprintf(&dat[strlen(dat)],"indev=%s ",ipt_info->indev);
	if ( strlen(ipt_info->outdev) >= 3)
		sprintf(&dat[strlen(dat)],"outdev=%s ",ipt_info->outdev);
*/
	if (ipt_info->protocol == 1 ) {
		sprintf(&dat[strlen(dat)],"ICMP type=%d code=%d len=%u",(int)ipt_info->icmp_type,(int)ipt_info->icmp_code,ipt_info->pktlen);
	} else {
		if (ipt_info->protocol == 17)
			sprintf(&dat[strlen(dat)],"UDP SPT=%u DPT=%u len=%u",ipt_info->sport,ipt_info->dport,ipt_info->pktlen);
		else if (ipt_info->protocol == 6) {
			char flags[32];
			memset(flags,0,32);
			get_pct_flags(ipt_info,flags);
			sprintf(&dat[strlen(dat)],"TCP SPT=%u DPT=%u",ipt_info->sport,ipt_info->dport);
			sprintf(&dat[strlen(dat)]," len=%u window=%u %s",ipt_info->pktlen, ipt_info->tcp_window,flags);
		} else 
			sprintf(&dat[strlen(dat)],"%d len=%u",ipt_info->protocol,ipt_info->pktlen);
	}

	writelog(LOG_INFO,dat);
}


