#!/usr/bin/env lua
--
--	stop_nfque.lua
--		2013.10.11
--		By H.Nakano
--  iptables -t mangle -n --line-numbers --list POSTROUTING
--  iptables -t raw -n --line-numbers --list PREROUTING
--

package.path = "/usr/local/share/lua/5.2/?.lua;" .. package.path

local data   = require("pl.data")
local tablex = require("pl.tablex")

local tbls = {"raw","mangle"}
local tbllists = {raw='PREROUTING',mangle='POSTROUTING'}

local function get_iptable(tbl)
	local cmd = string.format("iptables -t %s -n --line-numbers --list %s",tbl,tbllists[tbl])
	local f = io.popen(cmd)
	local s = data.read(f,{last_field_collect=true})
	f:close()

	if s == nil then 
		print("Error: iptable cmd")
		return nil
	end

	local val = {}
	first = 9999
	for n,v in ipairs(s) do
		if v[2] == "NFQUEUE" then
			table.insert(val,v[1])
			first = n
		else
			if n > first then
				break
			end
		end
	end
	if tablex.size(val) == 0 then
		print(string.format(" iptables (%s) no NFQUEUE",tbl))
		return nil
	else
		return val
	end
end

local function del_nfqueue(tbl)
		val = get_iptable(tbl)
		if val == nil then
			return 
		end

		first = 0
		count = 0
		for n,v in ipairs(val) do
			num = tonumber(v)
			if first == 0 then
				first = num
			else
				if num ~= first + count + 1 then
					print(string.format("Error:%s:%s",tbl,v))
					break
				end
				count = count + 1
			end
			print(string.format("OK delete:%s:%s",tbl,v))
			os.execute(string.format("iptables -t %s -D %s %d",tbl,tbllists[tbl],first))
		end
end

for _,v in ipairs(tbls) do
	 del_nfqueue(v)
end


