/********************************  
	tcp_client2.c
	2013.9.6
		By H.Nakano
************************************/ 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#include "hashdb.h"

/* socktools.c */
extern int send_big_data(int sockfd,char *buf,int size);
extern int recv_big_data(int sockfd,char *buf,int size);

/* write_log.c */
extern void writelog(int prio,char *fmt, ...);


// get sockaddr, IPv4 or IPv6:
void *get_in_addr(struct sockaddr *sa){
	if (sa->sa_family == AF_INET) {
		return &(((struct sockaddr_in*)sa)->sin_addr);
	}
	return &(((struct sockaddr_in6*)sa)->sin6_addr);
}

int tcp_client(tcp_data_t *val,char *buf){
	int sockfd, numbytes,rv;
	struct addrinfo hints, *servinfo, *p;
	net_access_t ndata;
	char s[INET6_ADDRSTRLEN];

	memset((void *)&ndata, 0, sizeof(net_access_t));
	memset((void *)&hints, 0, sizeof(hints));
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;

	if ((rv = getaddrinfo(val->host,TcpPort, &hints, &servinfo)) != 0) {
		writelog(LOG_ERR,"Error:tcp_client getaddrinfo(%s)",gai_strerror(rv));
		return -1;
	 }
	
	for(p = servinfo; p != NULL; p = p->ai_next) {
		errno = 0;
		if ((sockfd = socket(p->ai_family, p->ai_socktype,p->ai_protocol)) == -1) {
			writelog(LOG_ERR,"Error:tcp_client socket(%s)",strerror(errno));
			continue;
		}
		errno = 0;
		if (connect(sockfd, p->ai_addr, p->ai_addrlen) == -1) {
			close(sockfd);
			writelog(LOG_ERR,"Error:tcp_client connect(%s)",strerror(errno));
			continue;
		 }
		break;
	}

	if (p == NULL) {
		writelog(LOG_ERR,"Error:tcp_client failed to connect");
		return -1;
	}

	inet_ntop(p->ai_family, get_in_addr((struct sockaddr *)p->ai_addr),s, sizeof s);
	printf("client: connecting to %s\n", s);
	freeaddrinfo(servinfo); 

	ndata.flag = val->flag;
	ndata.size = val->size;
	memcpy((void *)ndata.passwd,val->passwd, sizeof(val->passwd));

	if (val->data != NULL && val->size == 0){
		ipaddr_range_t *rr = (ipaddr_range_t *)val->data;
		ndata.num   = rr->num;
		ndata.range = rr->range;
	 } else if ( val->flag == BLK_LIST){
		ndata.top = val->top;
	}

	errno = 0;
	if (send(sockfd,(void *)&ndata, sizeof(net_access_t), 0) == -1){
		writelog(LOG_ERR,"Error:tcp_client send data(%s)",strerror(errno));
		close(sockfd);
		return -1;
	 }

	// send-loop 
	if (val->size > 0){
		numbytes = send_big_data(sockfd,(void *)val->data,val->size*sizeof(ipaddr_range_t));
		if (numbytes == -1){
			close(sockfd);
			return -1;
	 	}
	 }

	// recv-loop
	if (ndata.flag == BLK_LIST || ndata.flag == BLK_SUM){
		errno = 0;
		if ((numbytes = recv(sockfd,s,sizeof(s), 0)) == -1){
			writelog(LOG_ERR,"Error:tcp_client recv data(%s)",strerror(errno));
			close(sockfd);
			return -2;
	 	 } 
		int size = atoi(s);
		if (size <= 0){
			strncpy(buf,s,numbytes);
			close(sockfd);
			return 0;
		 }
	//	printf("recv size:%d:(%d)\n", size,numbytes);
		val->data = (int *)malloc(size*sizeof(int));
		int ret = recv_big_data(sockfd,(void *)val->data,size*sizeof(int));
		if (ret == -1){
			free(val->data);
			close(sockfd);
			return -2;
	 	 }
		val->size = size;
		strcpy(buf,Result_OK);
		
	 } else{
		  errno = 0;
		  if ((numbytes = recv(sockfd, buf,INET6_ADDRSTRLEN, 0)) == -1)
				writelog(LOG_ERR,"Error:tcp_client recv data(%s)",strerror(errno));
	 	  else 
				buf[numbytes] = '\0';
	 }
	close(sockfd);
	return 0;
}



