/**************************
	tcp_server2.c
		2013.9.6
		By H.Nakano
**************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <stdbool.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include <syslog.h>

#include "hashdb.h"

/* write_log.c */
extern void writelog(int prio,char *fmt, ...);

/* hashdb.c */
extern int check_val (int flag,uint32_t num,int range);
extern char *check_val2(int flag,int top,int *size);
extern int check_val3(int flag,int size,ipaddr_range_t *rr);
/* msgS.c */
extern void stop_msg();

/* socktools.c */
extern int send_big_data(int sockfd,char *buf,int size);
extern int recv_big_data(int sockfd,char *buf,int size);

void sigchld_handler(int s){
	while(waitpid(-1, NULL, WNOHANG) > 0);
}

// get sockaddr, IPv4 or IPv6:
void *get_in_addr(struct sockaddr *sa){
	if (sa->sa_family == AF_INET) {
		return &(((struct sockaddr_in*)sa)->sin_addr);
	}
	return &(((struct sockaddr_in6*)sa)->sin6_addr);
}

static int create_and_bind(){
  	struct addrinfo hints;
  	struct addrinfo *servinfo, *p;
	int yes=1;
	int rv,sockfd;

 	memset(&hints, 0, sizeof hints);
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_flags = AI_PASSIVE; 

	errno = 0;
	if ((rv = getaddrinfo(NULL,TcpPort, &hints, &servinfo)) != 0) {
		writelog(LOG_ERR,"Error:tcp_server getaddrinfo(%s)", gai_strerror(rv));
		return -1;
	}

	for(p = servinfo; p != NULL; p = p->ai_next) {
		errno = 0;
		if ((sockfd = socket(p->ai_family, p->ai_socktype,p->ai_protocol)) == -1) {
			continue;
		 }
		errno = 0;
		if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &yes,sizeof(int)) == -1) {
			writelog(LOG_ERR,"Error:tcp_server setsockopt(%s)",strerror(errno));
			return -1;
		 }
		if (bind(sockfd, p->ai_addr, p->ai_addrlen) == -1) {
			close(sockfd);
			continue;
		 }
		break;
	}

	if (p == NULL) {
		writelog(LOG_ERR,"Error:tcp_server bind");
		return -1;
	}

	freeaddrinfo(servinfo);
  	return sockfd;
}

int check_host(allowed_host_t *hosts,char *s){
	
	for (int i =0; hosts[i].addr[0] != '\0';i++){ 
		if (strcmp(hosts[i].addr,s) == 0)
			return 0;
	  }
	return -1;
}

int check_passwd(allowed_host_t *hosts,char *s,net_access_t *ndata){

	for (int i =0; hosts[i].addr[0] != '\0';i++){
		if (strcmp(hosts[i].addr,s) == 0 && 
			strcmp(hosts[i].passwd,ndata->passwd) == 0 && (hosts[i].access & ndata->flag))
			return 0;
	  }
	return -1;
}

int tcp_server2(allowed_host_t *hosts){
	int sockfd, new_fd;
	int ret,numbytes;
	struct sockaddr_storage their_addr;
	socklen_t sin_size;
	struct sigaction sa;
	net_access_t ndata;
	char s[INET6_ADDRSTRLEN]; // 46
	bool quit = false;
	
 	sockfd = create_and_bind();
  	if (sockfd == -1)
    	return -1;

	errno = 0;
	if (listen(sockfd, BACKLOG) == -1) {
		writelog(LOG_ERR,"Error:tcp_server listen(%s)",strerror(errno));
		return -1;
	}

	sa.sa_handler = sigchld_handler;
	sigemptyset(&sa.sa_mask);
	sa.sa_flags = SA_RESTART;
	if (sigaction(SIGCHLD, &sa, NULL) == -1) {
		writelog(LOG_ERR,"Error:tcp_server sigaction(%s)",strerror(errno));
		return -1;
	}

	writelog(LOG_INFO,"tcp server: waiting for connections");
	while (!quit){
		errno = 0;
		sin_size = sizeof(their_addr);
		new_fd = accept(sockfd, (struct sockaddr *)&their_addr, &sin_size);
		if (new_fd == -1) {
			writelog(LOG_ERR,"Error:tcp_server accept(%s)",strerror(errno));
			continue;
		 }

		inet_ntop(their_addr.ss_family,get_in_addr((struct sockaddr *)&their_addr),s, sizeof s);
		if (check_host(hosts,s) < 0){
		//	shutdown(new_fd,2);
			close(new_fd);
			writelog(LOG_ERR,"Deny:tcp_server access:(%s)",s);
			continue;
		 }

		numbytes = recv(new_fd, (void *)&ndata, sizeof(net_access_t), 0);
		if ( numbytes == -1){ 
			writelog(LOG_ERR,"Error:tcp_server from=%s recv(%s)",s,strerror(errno));
		//	shutdown(new_fd,2);
			close(new_fd);
			continue;
		 }

		writelog(LOG_INFO,"tcp server: recv: form=%s flag=%d size=%d",s,ndata.flag,ndata.size);
		if (check_passwd(hosts,s,&ndata) < 0){
			strcpy(s,"Deny:bad passwd");
			if (send(new_fd,(void *)s,sizeof(s), 0) == -1)
				writelog(LOG_ERR,"Error:tcp_server send result(%s)",strerror(errno));
			close(new_fd);
			writelog(LOG_ERR,"Deny:tcp_server access:(%s) permission",s);
			continue;
		 }

		errno = 0;
		memset((void *)s,0,sizeof(s));
		if (ndata.size >= 1){
			numbytes = ndata.size*sizeof(ipaddr_range_t);
			char *buf = (void *)malloc(numbytes);
			numbytes = recv_big_data(new_fd,buf,numbytes);
			if (numbytes == -1){ 
			//	shutdown(new_fd,2);
				close(new_fd);
				free(buf);
				continue;
			 }
			/* hashdb.c */
			ret = check_val3(ndata.flag,ndata.size,(ipaddr_range_t *)buf);
			if (ret == Check_OK)
			  	strcpy(s,Result_OK);
			else
				strcpy(s,Result_Fail);
			if (send(new_fd,(void *)s,sizeof(s), 0) == -1)
				writelog(LOG_ERR,"Error:tcp_server send 3(%s)",strerror(errno));
			free(buf);

	     } else {
			if (ndata.flag == BLK_KILL){
				quit = true;
				stop_msg();
				strcpy(s,Result_OK);
				if (send(new_fd,(void *)s,sizeof(s), 0) == -1)
					writelog(LOG_ERR,"Error:tcp_server send 4(%s)",strerror(errno));

			}else if (ndata.flag == BLK_LIST || ndata.flag == BLK_SUM){
				int size;
				char *data = check_val2(ndata.flag,ndata.top,&size); 
				if (data != NULL && size >= 1){
					sprintf(s,"%d",size);
					if (send(new_fd,(void *)s,sizeof(s), 0) == -1){
						writelog(LOG_ERR,"Error:tcp_server send header(%s)",strerror(errno));
						size = 0;
				    } else {
						send_big_data(new_fd,(char *)data,size*sizeof(int));
					 }
				 }
				errno = 0;
				if (data == NULL){
					if (size == 0)
						strcpy(s,Result_NoData);
					else
						strcpy(s,Result_Fail);
					if (send(new_fd,(void *)s,sizeof(s), 0) == -1)
						writelog(LOG_ERR,"Error:tcp_server send result(%s)",strerror(errno));
				 }
				if (data != NULL)
					free(data);

			} else if (ndata.flag == BLK_CHECK){
				ret = check_val(ndata.flag,ndata.num,ndata.range);
				if (ret == BlackON)
			  		strcpy(s,Result_Blacked);
				else if (ret == BlackPart)
					strcpy(s,Result_Partial);
				else
					strcpy(s,Result_NoBlack);

				if (send(new_fd,(void *)s,sizeof(s), 0) == -1)
					writelog(LOG_ERR,"Error:tcp_server send check(%s)",strerror(errno));
			} else {
				ret = check_val(ndata.flag,ndata.num,ndata.range);
				if (ret == Check_OK)
			  		strcpy(s,Result_OK);
				else
					strcpy(s,Result_Fail);
				if (send(new_fd,(void *)s,sizeof(s), 0) == -1)
					writelog(LOG_ERR,"Error:tcp_server send result(%s)",strerror(errno));
			 }
	     }

		close(new_fd); 
	}
	close(sockfd);
	return 0;
}

