/************************
  	write_log.c
		2013.9.21
		By H.Nakano
	version 0.1
****************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <syslog.h>

#include "zlog.h"

#define		ZLOG_Key    1
#define		SYSLOG_Key  2
#define		Bsize  		 1024

/* daemon flag */
static int dflag = 0;
static int pflag = 0;
static zlog_category_t *zc = NULL;

static void vsend_log(int prio, const char *val) {

	switch(prio){
			case LOG_DEBUG:   zlog_debug(zc,val);  break;
			case LOG_INFO:    zlog_info(zc, val);  break;
			case LOG_NOTICE:  zlog_notice(zc,val); break;  
			case LOG_WARNING: zlog_warn(zc, val);  break; 
			case LOG_ERR:     zlog_error(zc, val); break; 
			case LOG_CRIT:    zlog_fatal(zc, val); break; 
			case LOG_ALERT:   zlog_fatal(zc, val); break; 
			case LOG_EMERG:   zlog_fatal(zc, val); break; 
			default:          zlog_info(zc, val);  break;
	}
}

void writelog(int prio,char *fmt, ...){
    va_list ap;
    va_start(ap, fmt);
	if (pflag == SYSLOG_Key)
		vsyslog(prio, fmt, ap);
	
//	if (zc != NULL || dflag == 0){
		char buf[Bsize];
		vsnprintf(buf, Bsize, fmt, ap);
		if (pflag == ZLOG_Key)
			vsend_log(prio,buf);
		if (dflag == 0)
   			fprintf(stdout,"%s\n",buf);
//	 }
    va_end(ap);
}

void close_log(){
	if (pflag == SYSLOG_Key)
		closelog();
	else if (pflag == ZLOG_Key)
		zlog_fini();
	pflag = 0;
}

/* set daemon flag */
void set_daemon(){
	dflag = 1;
}

/* syslog */
void open_log(char *keyname,int facility){
	openlog(keyname,LOG_ODELAY | LOG_PID,facility);
	pflag = SYSLOG_Key;
}

/* zlog */
int open_zlog(char *confile,char *key){
	int rc = zlog_init(confile);
	if (rc) {
		if (dflag)
			writelog( LOG_ERR,"zlog_init(%s) failed.\n",confile);
		else
			fprintf(stderr,"zlog_init(%s) failed.\n",confile);
		dflag = 0;
		return -1;
	}
	
	zc = zlog_get_category(key);
	if (!zc) {
		if (dflag)
		 	writelog( LOG_ERR,"zlog category(%s) failed.\n",key);
		else
			fprintf(stderr,"zlog category(%s) failed.\n",key);
		zlog_fini();
		dflag = 0;
		return -1;
	 } else{
		if (pflag == SYSLOG_Key)
			closelog();
		pflag = ZLOG_Key;
	 }
	return 0;
}


